@@ -59,14 +59,19 @@ IDebugSession* IBreakpointController::debugSession() const
 
 BreakpointModel* IBreakpointController::breakpointModel() const
 {
+    if (!ICore::self()) return 0;
     return ICore::self()->debugController()->breakpointModel();
 }
 
 void IBreakpointController::debuggerStateChanged(IDebugSession::DebuggerState state)
 {
+    BreakpointModel* model = breakpointModel();
+    if (!model)
+        return;
+
     if (state == IDebugSession::StartingState || state == IDebugSession::EndedState) {
         //breakpoint state changes when session started or stopped
-        foreach (Breakpoint *breakpoint, breakpointModel()->breakpoints()) {
+        foreach (Breakpoint *breakpoint, model->breakpoints()) {
             if (state == IDebugSession::StartingState) {
                 //when starting everything is dirty
                 m_dirty[breakpoint].insert(Breakpoint::LocationColumn);
@@ -81,7 +86,11 @@ void IBreakpointController::debuggerStateChanged(IDebugSession::DebuggerState st
 
 void IBreakpointController::sendMaybeAll()
 {
-    foreach (Breakpoint *breakpoint, breakpointModel()->breakpoints()) {
+    BreakpointModel* model = breakpointModel();
+    if (!model)
+        return;
+
+    foreach (Breakpoint *breakpoint, model->breakpoints()) {
         sendMaybe(breakpoint);
     }
 }
