@@ -24,7 +24,6 @@
 
 #include "ducontext.h"
 #include <language/util/setrepository.h>
-#include "indexeditems.h"
 #include <QMetaType>
 
 template< class T >
@@ -169,17 +168,32 @@ public:
 
   /**
    * Add a parsing-problem to this context.
+   *
+   * \note you must be holding a write lock when you access this function.
    * */
   void addProblem(const ProblemPointer& problem);
 
   /**
    * Remove a parsing-problem from this context.
+   *
+   * \note you must be holding a write lock when you access this function.
    * */
   void removeProblem(const ProblemPointer& problem);
 
-  /// Clear the list of problems
+  /**
+   * Clear the list of problems
+   *
+   * \note you must be holding a write lock when you access this function.
+   */
   void clearProblems();
 
+  /**
+   * Set the list of problems, replacing all existing ones.
+   *
+   * \note you must be holding a write lock when you access this function.
+   */
+  void setProblems(const QList<ProblemPointer>& pointers);
+
   /**
    * Determine if this chain imports another chain recursively.
    * 
@@ -190,16 +204,6 @@ public:
    */
   virtual bool imports(const DUContext* origin, const CursorInRevision& position) const;
 
-  /**
-   * Returns the trace of imports from this context top the given target.
-   * The positions in the returned trace may be invalid.
-   * This is more efficient then the version below, because the trace doesn't need to be copied
-   * */
-//   void importTrace(const TopDUContext* target, ImportTrace& store) const;
-
-  ///More convenient version of the above
-//   ImportTrace importTrace(const TopDUContext* target) const;
-
   enum {
     Identity = 4
   };
