@@ -18,6 +18,7 @@
 #include <QFileInfo>
 #include <QTextCodec>
 #include <QTextStream>
+#include <QDirIterator>
 
 #include <ktar.h>
 #include <kzip.h>
@@ -63,21 +64,15 @@ using KDevelop::IDistributedVersionControl;
 using KDevelop::IPlugin;
 using KDevelop::VcsJob;
 using KDevelop::VcsLocation;
+using KDevelop::ICore;
 
-K_PLUGIN_FACTORY(AppWizardFactory,
-    registerPlugin<AppWizardPlugin>();
-    KComponentData compData = componentData();
-    KStandardDirs *dirs = compData.dirs();
-    dirs->addResourceType("apptemplates", "data", "kdevappwizard/templates/");
-    dirs->addResourceType("apptemplate_descriptions","data", "kdevappwizard/template_descriptions/");
-    dirs->addResourceType("apptemplate_previews","data", "kdevappwizard/template_previews/");
-    setComponentData(compData);
-)
+K_PLUGIN_FACTORY(AppWizardFactory, registerPlugin<AppWizardPlugin>();)
 K_EXPORT_PLUGIN(AppWizardFactory(KAboutData("kdevappwizard","kdevappwizard", ki18n("Project Wizard"), "0.1", ki18n("Support for creating and importing projects"), KAboutData::License_GPL)))
 
 AppWizardPlugin::AppWizardPlugin(QObject *parent, const QVariantList &)
     :KDevelop::IPlugin(AppWizardFactory::componentData(), parent)
 {
+    KDEV_USE_EXTENSION_INTERFACE(KDevelop::ITemplateProvider);
     setXMLFile("kdevappwizard.rc");
 
     m_newFromTemplate = actionCollection()->addAction("project_new");
@@ -116,9 +111,9 @@ void AppWizardPlugin::slotNewProject()
             {
                 file = KMacroExpander::expandMacros(file, m_variables);
                 core()->documentController()->openDocument(file);
-            }                        
+            }
         } else {
-            KMessageBox::error( KDevelop::ICore::self()->uiController()->activeMainWindow(), i18n("Could not create project from template\n"), i18n("Failed to create project") );
+            KMessageBox::error( ICore::self()->uiController()->activeMainWindow(), i18n("Could not create project from template\n"), i18n("Failed to create project") );
         }
     }
 }
@@ -216,23 +211,20 @@ QString generateIdentifier( const QString& appname )
 QString AppWizardPlugin::createProject(const ApplicationInfo& info)
 {
     QFileInfo templateInfo(info.appTemplate);
-    if (!templateInfo.exists())
+    if (!templateInfo.exists()) {
+        kWarning() << "Project app template does not exist:" << info.appTemplate;
         return QString();
+    }
 
     QString templateName = templateInfo.baseName();
     kDebug() << "creating project for template:" << templateName << " with VCS:" << info.vcsPluginName;
 
-    QString templateArchive = componentData().dirs()->findResource("apptemplates", templateName + ".zip");
-    if( templateArchive.isEmpty() )
-    {
-        templateArchive = componentData().dirs()->findResource("apptemplates", templateName + ".tar.bz2");
+    QStringList matches = ICore::self()->componentData().dirs()->findAllResources("data", QString("kdevappwizard/templates/%1*").arg(templateName));
+    if (matches.isEmpty()) {
+        kWarning() << "Could not find project template" << templateName;
+        return QString();;
     }
-
-    kDebug() << "Using archive:" << templateArchive;
-
-    if (templateArchive.isEmpty())
-        return QString();
-
+    QString templateArchive = matches.first();
 
     KUrl dest = info.location;
 
@@ -326,6 +318,15 @@ QString AppWizardPlugin::createProject(const ApplicationInfo& info)
 
     QString projectFileName = QDir::cleanPath( dest.toLocalFile() + '/' + info.name + ".kdev4" );
 
+    // Loop through the new project directory and try to detect the first .kdev4 file.
+    // If one is found this file will be used. So .kdev4 file can be stored in any subdirectory and the
+    // project templates can be more complex.
+    QDirIterator it(QDir::cleanPath( dest.toLocalFile()), QStringList() << "*.kdev4", QDir::NoFilter, QDirIterator::Subdirectories);
+    if(it.hasNext() == true)
+    {
+        projectFileName = it.next();
+    }
+
     kDebug() << "Returning" << projectFileName << QFileInfo( projectFileName ).exists() ;
 
     if( ! QFileInfo( projectFileName ).exists() )
@@ -382,7 +383,7 @@ bool AppWizardPlugin::unpackArchive(const KArchiveDirectory *dir, const QString
         if (dir->entry(entry)->isDirectory())
         {
             const KArchiveDirectory *file = (KArchiveDirectory *)dir->entry(entry);
-            QString newdest = dest + '/' + file->name();
+            QString newdest = dest + '/' + KMacroExpander::expandMacros(file->name(), m_variables);
             if( !QFileInfo( newdest ).exists() )
             {
                 QDir::root().mkdir( newdest  );
@@ -458,4 +459,44 @@ KDevelop::ContextMenuExtension AppWizardPlugin::contextMenuExtension(KDevelop::C
     return ext;
 }
 
+QAbstractItemModel* AppWizardPlugin::templatesModel() const
+{
+    return m_templatesModel;
+}
+
+QString AppWizardPlugin::knsConfigurationFile() const
+{
+    return "kdevappwizard.knsrc";
+}
+
+QStringList AppWizardPlugin::supportedMimeTypes() const
+{
+    QStringList types;
+    types << "application/x-desktop";
+    types << "application/x-bzip-compressed-tar";
+    types << "application/zip";
+    return types;
+}
+
+QIcon AppWizardPlugin::icon() const
+{
+    return KIcon("project-development-new-template");
+}
+
+QString AppWizardPlugin::name() const
+{
+    return i18n("Project Templates");
+}
+
+void AppWizardPlugin::loadTemplate(const QString& fileName)
+{
+    m_templatesModel->loadTemplateFile(fileName);
+}
+
+void AppWizardPlugin::reload()
+{
+    m_templatesModel->refresh();
+}
+
+
 #include "appwizardplugin.moc"
