@@ -47,7 +47,7 @@ NativeAppJob::NativeAppJob(QObject* parent, KDevelop::ILaunchConfiguration* cfg)
     kDebug() << "creating native app job";
     setCapabilities(Killable);
     
-    IExecutePlugin* iface = KDevelop::ICore::self()->pluginController()->pluginForExtension("org.kdevelop.IExecutePlugin")->extension<IExecutePlugin>();
+    IExecutePlugin* iface = KDevelop::ICore::self()->pluginController()->pluginForExtension("org.kdevelop.IExecutePlugin", "kdevexecute")->extension<IExecutePlugin>();
     Q_ASSERT(iface);
         
     KDevelop::EnvironmentGroupList l(KGlobal::config());
@@ -90,7 +90,7 @@ NativeAppJob::NativeAppJob(QObject* parent, KDevelop::ILaunchConfiguration* cfg)
     
     setStandardToolView(KDevelop::IOutputView::RunView);
     setBehaviours(KDevelop::IOutputView::AllowUserClose | KDevelop::IOutputView::AutoScroll);
-    setModel( new KDevelop::OutputModel(), KDevelop::IOutputView::TakeOwnership );
+    setModel( new KDevelop::OutputModel );
     
     connect( lineMaker, SIGNAL(receivedStdoutLines(QStringList)), model(), SLOT(appendLines(QStringList)) );
     connect( proc, SIGNAL(error(QProcess::ProcessError)), SLOT(processError(QProcess::ProcessError)) );
@@ -160,16 +160,18 @@ void NativeAppJob::processFinished( int exitCode , QProcess::ExitStatus status )
 {
     lineMaker->flushBuffers();
     
-    if (exitCode == 0 && status == QProcess::NormalExit)
+    if (exitCode == 0 && status == QProcess::NormalExit) {
         appendLine( i18n("*** Exited normally ***") );
-    else
-        if (status == QProcess::NormalExit)
-            appendLine( i18n("*** Exited with return code: %1 ***", QString::number(exitCode)) );
-        else 
-            if (error() == KJob::KilledJobError)
-                appendLine( i18n("*** Process aborted ***") );
-            else
-                appendLine( i18n("*** Crashed with return code: %1 ***", QString::number(exitCode)) );
+    } else if (status == QProcess::NormalExit) {
+        appendLine( i18n("*** Exited with return code: %1 ***", QString::number(exitCode)) );
+        setError(OutputJob::FailedShownError);
+    } else if (error() == KJob::KilledJobError) {
+        appendLine( i18n("*** Process aborted ***") );
+        setError(KJob::KilledJobError);
+    } else {
+        appendLine( i18n("*** Crashed with return code: %1 ***", QString::number(exitCode)) );
+        setError(OutputJob::FailedShownError);
+    }
     kDebug() << "Process done";
     emitResult();
 }
