@@ -42,6 +42,7 @@
 #include <vcs/vcsannotation.h>
 #include <vcs/widgets/standardvcslocationwidget.h>
 #include <KIO/CopyJob>
+#include <KIO/NetAccess>
 #include "gitclonejob.h"
 #include <interfaces/contextmenuextension.h>
 #include <QMenu>
@@ -49,13 +50,25 @@
 #include "stashmanagerdialog.h"
 #include <KMessageBox>
 #include <KStandardDirs>
+#include <KTextEdit>
 #include "gitjob.h"
+#include "gitmessagehighlighter.h"
 
 K_PLUGIN_FACTORY(KDevGitFactory, registerPlugin<GitPlugin>(); )
 K_EXPORT_PLUGIN(KDevGitFactory(KAboutData("kdevgit","kdevgit",ki18n("Git"),"0.1",ki18n("A plugin to support git version control systems"), KAboutData::License_GPL)))
 
 using namespace KDevelop;
 
+QVariant runSynchronously(KDevelop::VcsJob* job)
+{
+    QVariant ret;
+    if(job->exec() && job->status()==KDevelop::VcsJob::JobSucceeded) {
+        ret = job->fetchResults();
+    }
+    delete job;
+    return ret;
+}
+
 namespace
 {
     
@@ -125,8 +138,6 @@ QString toRevisionName(const KDevelop::VcsRevision& rev, QString currentRevision
 QString revisionInterval(const KDevelop::VcsRevision& rev, const KDevelop::VcsRevision& limit)
 {
     QString ret;
-//     qDebug() << "prrrrrrrrrr" << toRevisionName(rev, "xxx") << toRevisionName(limit, "yyy");
-    
     if(rev.revisionType()==VcsRevision::Special &&
                 rev.revisionValue().value<VcsRevision::RevisionSpecialType>()==VcsRevision::Start) //if we want it to the begining just put the revisionInterval
         ret = toRevisionName(limit, QString());
@@ -142,8 +153,6 @@ QString revisionInterval(const KDevelop::VcsRevision& rev, const KDevelop::VcsRe
                 ret = src+".."+dst;
         }
     }
-    
-//     qDebug() << "=======>" << ret;
     return ret;
 }
 
@@ -170,6 +179,7 @@ GitPlugin::GitPlugin( QObject *parent, const QVariantList & )
 
     KDEV_USE_EXTENSION_INTERFACE( KDevelop::IBasicVersionControl )
     KDEV_USE_EXTENSION_INTERFACE( KDevelop::IDistributedVersionControl )
+    KDEV_USE_EXTENSION_INTERFACE( KDevelop::IBranchingVersionControl )
 
     m_hasError = false;
     core()->uiController()->addToolView(i18n("Git"), dvcsViewFactory());
@@ -208,12 +218,11 @@ void GitPlugin::additionalMenuEntries(QMenu* menu, const KUrl::List& urls)
     m_urls = urls;
     
     QDir dir=urlDir(urls);
-    bool modif = hasModifications(dotGitDirectory(urls.first()));
-    bool canApply = !modif && hasStashes(dir);
+    bool hasSt = hasStashes(dir);
     menu->addSeparator()->setText(i18n("Git Stashes"));
-    menu->addAction(i18n("Stash Manager"), this, SLOT(ctxStashManager()))->setEnabled(canApply);
-    menu->addAction(i18n("Push Stash"), this, SLOT(ctxPushStash()))->setEnabled(modif);
-    menu->addAction(i18n("Pop Stash"), this, SLOT(ctxPopStash()))->setEnabled(canApply);
+    menu->addAction(i18n("Stash Manager"), this, SLOT(ctxStashManager()))->setEnabled(hasSt);
+    menu->addAction(i18n("Push Stash"), this, SLOT(ctxPushStash()));
+    menu->addAction(i18n("Pop Stash"), this, SLOT(ctxPopStash()))->setEnabled(hasSt);
 }
 
 void GitPlugin::ctxPushStash()
@@ -269,6 +278,9 @@ bool GitPlugin::isValidDirectory(const KUrl & dirPath)
 bool GitPlugin::isVersionControlled(const KUrl &path)
 {
     QFileInfo fsObject(path.toLocalFile());
+    if (!fsObject.exists()) {
+        return false;
+    }
     if (fsObject.isDir()) {
         return isValidDirectory(path);
     }
@@ -282,6 +294,7 @@ bool GitPlugin::isVersionControlled(const KUrl &path)
 VcsJob* GitPlugin::init(const KUrl &directory)
 {
     DVcsJob* job = new DVcsJob(urlDir(directory), this);
+    job->setType(VcsJob::Import);
     *job << "git" << "init";
     return job;
 }
@@ -289,6 +302,7 @@ VcsJob* GitPlugin::init(const KUrl &directory)
 VcsJob* GitPlugin::createWorkingCopy(const KDevelop::VcsLocation & source, const KUrl& dest, KDevelop::IBasicVersionControl::RecursionMode)
 {
     DVcsJob* job = new GitCloneJob(urlDir(dest), this);
+    job->setType(VcsJob::Import);
     *job << "git" << "clone" << "--progress" << "--" << source.localUrl().url() << dest;
     return job;
 }
@@ -299,6 +313,7 @@ VcsJob* GitPlugin::add(const KUrl::List& localLocations, KDevelop::IBasicVersion
         return errorsFound(i18n("Did not specify the list of files"), OutputJob::Verbose);
 
     DVcsJob* job = new GitJob(dotGitDirectory(localLocations.front()), this);
+    job->setType(VcsJob::Add);
     *job << "git" << "add" << "--" << (recursion == IBasicVersionControl::Recursive ? localLocations : preventRecursion(localLocations));
     return job;
 }
@@ -309,6 +324,7 @@ KDevelop::VcsJob* GitPlugin::status(const KUrl::List& localLocations, KDevelop::
         return errorsFound(i18n("Did not specify the list of files"), OutputJob::Verbose);
 
     DVcsJob* job = new GitJob(urlDir(localLocations), this, OutputJob::Silent);
+    job->setType(VcsJob::Status);
     
     if(m_oldVersion) {
         *job << "git" << "ls-files" << "-t" << "-m" << "-c" << "-o" << "-d" << "-k" << "--directory";
@@ -323,15 +339,23 @@ KDevelop::VcsJob* GitPlugin::status(const KUrl::List& localLocations, KDevelop::
 }
 
 VcsJob* GitPlugin::diff(const KUrl& fileOrDirectory, const KDevelop::VcsRevision& srcRevision, const KDevelop::VcsRevision& dstRevision,
-                        VcsDiff::Type type, IBasicVersionControl::RecursionMode recursion)
+                        VcsDiff::Type /*type*/, IBasicVersionControl::RecursionMode recursion)
 {
     //TODO: control different types
     
-    DVcsJob* job = new GitJob(dotGitDirectory(fileOrDirectory), this);
+    DVcsJob* job = new GitJob(dotGitDirectory(fileOrDirectory), this, KDevelop::OutputJob::Silent);
+    job->setType(VcsJob::Diff);
     *job << "git" << "diff" << "--no-prefix" << "--no-color" << "--no-ext-diff";
-    QString revstr = revisionInterval(srcRevision, dstRevision);
-    if(!revstr.isEmpty())
-        *job << revstr;
+    if(srcRevision.revisionType()==VcsRevision::Special
+        && dstRevision.revisionType()==VcsRevision::Special
+        && srcRevision.specialType()==VcsRevision::Base
+        && dstRevision.specialType()==VcsRevision::Working)
+        *job << "HEAD";
+    else {
+        QString revstr = revisionInterval(srcRevision, dstRevision);
+        if(!revstr.isEmpty())
+            *job << revstr;
+    }
     
     *job << "--" << (recursion == IBasicVersionControl::Recursive ? fileOrDirectory : preventRecursion(fileOrDirectory));
     
@@ -345,6 +369,7 @@ VcsJob* GitPlugin::revert(const KUrl::List& localLocations, IBasicVersionControl
         return errorsFound(i18n("Could not revert changes"), OutputJob::Verbose);
     
     DVcsJob* job = new GitJob(dotGitDirectory(localLocations.front()), this);
+    job->setType(VcsJob::Revert);
     *job << "git" << "checkout" << "--";
     *job << (recursion == IBasicVersionControl::Recursive ? localLocations : preventRecursion(localLocations));
     
@@ -363,6 +388,7 @@ VcsJob* GitPlugin::commit(const QString& message,
 
     QDir dir = dotGitDirectory(localLocations.front());
     DVcsJob* job = new DVcsJob(dir, this);
+    job->setType(VcsJob::Commit);
     KUrl::List files = (recursion == IBasicVersionControl::Recursive ? localLocations : preventRecursion(localLocations));
     addNotVersionedFiles(dir, files);
     
@@ -395,28 +421,75 @@ void GitPlugin::addNotVersionedFiles(const QDir& dir, const KUrl::List& files)
     }
 }
 
+bool isEmptyDirStructure(const QDir &dir)
+{
+    foreach (const QFileInfo &i, dir.entryInfoList(QDir::AllEntries | QDir::NoDotAndDotDot)) {
+        if (i.isDir()) {
+            if (!isEmptyDirStructure(QDir(i.filePath()))) return false;
+        } else if (i.isFile()) {
+            return false;
+        }
+    }
+    return true;
+}
+
 VcsJob* GitPlugin::remove(const KUrl::List& files)
 {
     if (files.isEmpty())
         return errorsFound(i18n("No files to remove"));
-    QDir dir = dotGitDirectory(files.front());
+    QDir dotGitDir = dotGitDirectory(files.front());
 
-    QStringList otherStr = getLsFiles(dir, QStringList() << "--others" << "--" << files.front().toLocalFile(), KDevelop::OutputJob::Silent);
-    if(otherStr.isEmpty()) {
-        DVcsJob* job = new GitJob(dir, this);
-        *job << "git" << "rm" << "-r";
-        *job << "--" << files;
-        return job;
-    } else {
-        return new StandardJob(this, KIO::trash(files), KDevelop::OutputJob::Silent);
+
+    KUrl::List files_(files);
+
+    QMutableListIterator<KUrl> i(files_);
+    while (i.hasNext()) {
+        KUrl file = i.next();
+        QFileInfo fileInfo(file.toLocalFile());
+
+        QStringList otherStr = getLsFiles(dotGitDir, QStringList() << "--others" << "--" << file.toLocalFile(), KDevelop::OutputJob::Silent);
+        kDebug() << "other files" << otherStr;
+        if(!otherStr.isEmpty()) {
+            //remove files not under version control
+            KUrl::List otherFiles;
+            foreach(const QString &f, otherStr) {
+                otherFiles << KUrl::fromLocalFile(dotGitDir.path()+'/'+f);
+            }
+            if (fileInfo.isFile()) {
+                //if it's an unversioned file we are done, don't use git rm on it
+                i.remove();
+            }
+            KIO::NetAccess::synchronousRun(KIO::trash(otherFiles), 0);
+        }
+
+        if (fileInfo.isDir()) {
+            if (isEmptyDirStructure(QDir(file.toLocalFile()))) {
+                //remove empty folders, git doesn't do that
+                kDebug() << "empty folder, removing" << file;
+                KIO::NetAccess::synchronousRun(KIO::trash(file), 0);
+                //we already deleted it, don't use git rm on it
+                i.remove();
+            }
+        }
     }
+
+    if (files_.isEmpty()) return 0;
+
+    DVcsJob* job = new GitJob(dotGitDir, this);
+    job->setType(VcsJob::Remove);
+    // git refuses to delete files with local modifications
+    // use --force to overcome this
+    *job << "git" << "rm" << "-r" << "--force";
+    *job << "--" << files_;
+    return job;
 }
 
 VcsJob* GitPlugin::log(const KUrl& localLocation,
                 const KDevelop::VcsRevision& src, const KDevelop::VcsRevision& dst)
 {
-    DVcsJob* job = new GitJob(dotGitDirectory(localLocation), this);
-    *job << "git" << "log" << "--date=raw";
+    DVcsJob* job = new GitJob(dotGitDirectory(localLocation), this, KDevelop::OutputJob::Silent);
+    job->setType(VcsJob::Log);
+    *job << "git" << "log" << "--date=raw" /*<< "--numstat"*/;
     QString rev = revisionInterval(dst, src);
     if(!rev.isEmpty())
         *job << rev;
@@ -429,7 +502,8 @@ VcsJob* GitPlugin::log(const KUrl& localLocation,
 VcsJob* GitPlugin::log(const KUrl& localLocation, const KDevelop::VcsRevision& rev, unsigned long int limit)
 {
     DVcsJob* job = new GitJob(dotGitDirectory(localLocation), this, KDevelop::OutputJob::Silent);
-    *job << "git" << "log" << "--date=raw" << toRevisionName(rev, QString());
+    job->setType(VcsJob::Log);
+    *job << "git" << "log" << "--date=raw" /*<< "--numstat"*/ << toRevisionName(rev, QString());
     if(limit>0)
         *job << QString("-%1").arg(limit);
     
@@ -441,7 +515,8 @@ VcsJob* GitPlugin::log(const KUrl& localLocation, const KDevelop::VcsRevision& r
 KDevelop::VcsJob* GitPlugin::annotate(const KUrl &localLocation, const KDevelop::VcsRevision&)
 {
     DVcsJob* job = new GitJob(dotGitDirectory(localLocation), this, KDevelop::OutputJob::Silent);
-    *job << "git" << "blame" << "--porcelain";
+    job->setType(VcsJob::Annotate);
+    *job << "git" << "blame" << "--porcelain" << "-w";
     *job << "--" << localLocation;
     connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), this, SLOT(parseGitBlameOutput(KDevelop::DVcsJob*)));
     return job;
@@ -450,7 +525,7 @@ KDevelop::VcsJob* GitPlugin::annotate(const KUrl &localLocation, const KDevelop:
 void GitPlugin::parseGitBlameOutput(DVcsJob *job)
 {
     QVariantList results;
-    VcsAnnotationLine* annotation;
+    VcsAnnotationLine* annotation = 0;
     QStringList lines = job->output().split('\n');
     
     bool skipNext=false;
@@ -506,33 +581,6 @@ void GitPlugin::parseGitBlameOutput(DVcsJob *job)
     job->setResults(results);
 }
 
-DVcsJob* GitPlugin::switchBranch(const QString &repository, const QString &branch)
-{
-    QDir d(repository);
-    
-    if(hasModifications(d) && KMessageBox::questionYesNo(0, i18n("There are pending changes, do you want to stash them first?"))==KMessageBox::Yes) {
-        QScopedPointer<DVcsJob> stash(gitStash(d, QStringList(), KDevelop::OutputJob::Verbose));
-        stash->exec();
-    }
-    
-    DVcsJob* job = new DVcsJob(d, this);
-    *job << "git" << "checkout" << branch;
-    return job;
-}
-
-DVcsJob* GitPlugin::branch(const QString &repository, const QString &basebranch, const QString &branch,
-                             const QStringList &args)
-{
-    DVcsJob* job = new DVcsJob(QDir(repository), this, KDevelop::OutputJob::Silent);
-    *job << "git" << "branch" << args;
-    
-    *job << "--";
-    if (!branch.isEmpty())
-        *job << branch;
-    if (!basebranch.isEmpty())
-        *job << basebranch;
-    return job;
-}
 
 DVcsJob* GitPlugin::lsFiles(const QDir &repository, const QStringList &args,
                             OutputJob::OutputJobVerbosity verbosity)
@@ -549,48 +597,98 @@ DVcsJob* GitPlugin::gitStash(const QDir& repository, const QStringList& args, Ou
     return job;
 }
 
-QString GitPlugin::curBranch(const QString &repository)
+VcsJob* GitPlugin::tag(const KUrl& repository, const QString& commitMessage, const VcsRevision& rev, const QString& tagName)
 {
-    kDebug() << "Getting branch list";
-    
-    QScopedPointer<DVcsJob> job(new DVcsJob(QDir(repository), this, OutputJob::Silent));
-    *job << "git" << "symbolic-ref" << "HEAD";
-    if (job->exec() && job->status() == KDevelop::VcsJob::JobSucceeded) {
-        QString out = job->output().trimmed();
-        
-        kDebug() << "Getting branch list" << out.right(out.size()-11);
-        return out.right(out.size()-11);
-    }
-    return QString();
+    DVcsJob* job = new DVcsJob(urlDir(repository), this);
+    *job << "git" << "tag" << "-m" << commitMessage << tagName;
+    if(rev.revisionValue().isValid())
+        *job << rev.revisionValue().toString();
+    return job;
 }
 
-QStringList GitPlugin::branches(const QString &repository)
+VcsJob* GitPlugin::switchBranch(const KUrl &repository, const QString &branch)
 {
-    QStringList branchListDirty;
-    QScopedPointer<DVcsJob> job(branch(repository));
-    kDebug() << "Getting branch list";
+    QDir d=urlDir(repository);
     
-    if (job->exec() && job->status() == KDevelop::VcsJob::JobSucceeded)
-        branchListDirty = job->output().split('\n', QString::SkipEmptyParts);
-    else
-        return QStringList();
+    if(hasModifications(d) && KMessageBox::questionYesNo(0, i18n("There are pending changes, do you want to stash them first?"))==KMessageBox::Yes) {
+        QScopedPointer<DVcsJob> stash(gitStash(d, QStringList(), KDevelop::OutputJob::Verbose));
+        stash->exec();
+    }
+    
+    DVcsJob* job = new DVcsJob(d, this);
+    *job << "git" << "checkout" << branch;
+    return job;
+}
+
+VcsJob* GitPlugin::branch(const KUrl& repository, const KDevelop::VcsRevision& rev, const QString& branchName)
+{
+    Q_ASSERT(!branchName.isEmpty());
+    
+    DVcsJob* job = new DVcsJob(urlDir(repository), this);
+    *job << "git" << "branch" << "--" << branchName;
+    
+    if(!rev.prettyValue().isEmpty())
+        *job << rev.revisionValue().toString();
+    return job;
+}
+
+VcsJob* GitPlugin::deleteBranch(const KUrl& repository, const QString& branchName)
+{
+    DVcsJob* job = new DVcsJob(urlDir(repository), this, OutputJob::Silent);
+    *job << "git" << "branch" << "-D" << branchName;
+    connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitCurrentBranch(KDevelop::DVcsJob*)));
+    return job;
+}
+
+VcsJob* GitPlugin::renameBranch(const KUrl& repository, const QString& oldBranchName, const QString& newBranchName)
+{
+    DVcsJob* job = new DVcsJob(urlDir(repository), this, OutputJob::Silent);
+    *job << "git" << "branch" << "-m" << newBranchName << oldBranchName;
+    connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitCurrentBranch(KDevelop::DVcsJob*)));
+    return job;
+}
+
+VcsJob* GitPlugin::currentBranch(const KUrl& repository)
+{
+    DVcsJob* job = new DVcsJob(urlDir(repository), this, OutputJob::Silent);
+    *job << "git" << "symbolic-ref" << "HEAD";
+    connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitCurrentBranch(KDevelop::DVcsJob*)));
+    return job;
+}
+
+void GitPlugin::parseGitCurrentBranch(DVcsJob* job)
+{
+    QString out = job->output().trimmed();
+    job->setResults(out.right(out.size()-11));
+}
+
+VcsJob* GitPlugin::branches(const KUrl &repository)
+{
+    DVcsJob* job=new DVcsJob(urlDir(repository));
+    *job << "git" << "branch" << "-a";
+    connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitBranchOutput(KDevelop::DVcsJob*)));
+    return job;
+}
+
+void GitPlugin::parseGitBranchOutput(DVcsJob* job)
+{
+    QStringList branchListDirty = job->output().split('\n', QString::SkipEmptyParts);
 
     QStringList branchList;
     foreach(QString branch, branchListDirty)
     {
-        if (branch.contains('*'))
-        {
-            branch = branch.prepend('\n').section("\n*", 1);
-        }
-        else
-        {
-            branch = branch.prepend('\n').section('\n', 1);
-        }
+        // Skip pointers to another branches (one example of this is "origin/HEAD -> origin/master");
+        // "git rev-list" chokes on these entries and we do not need duplicate branches altogether.
+        if (branch.contains("->"))
+            continue;
         
-        branch = branch.trimmed();
-        branchList<<branch;
+        if (branch.startsWith('*'))
+            branch = branch.right(branch.size()-2);
+        
+        branchList<<branch.trimmed();
     }
-    return branchList;
+    
+    job->setResults(branchList);
 }
 
 /* Few words about how this hardcore works:
@@ -623,6 +721,7 @@ QList<DVcsEvent> GitPlugin::getAllCommits(const QString &repo)
     QScopedPointer<DVcsJob> job(gitRevList(repo, args));
     bool ret = job->exec();
     Q_ASSERT(ret && job->status()==VcsJob::JobSucceeded && "TODO: provide a fall back in case of failing");
+    Q_UNUSED(ret);
     QStringList commits = job->output().split('\n', QString::SkipEmptyParts);
 
     static QRegExp rx_com("commit \\w{40,40}");
@@ -779,23 +878,24 @@ QList<DVcsEvent> GitPlugin::getAllCommits(const QString &repo)
 
 void GitPlugin::initBranchHash(const QString &repo)
 {
-    QStringList branches = GitPlugin::branches(repo);
-    kDebug() << "BRANCHES: " << branches;
+    QStringList gitBranches = runSynchronously(branches(KUrl(repo))).toStringList();
+    kDebug() << "BRANCHES: " << gitBranches;
     //Now root branch is the current branch. In future it should be the longest branch
     //other commitLists are got with git-rev-lits branch ^br1 ^ br2
-    QString root = GitPlugin::curBranch(repo);
+    QString root = runSynchronously(currentBranch(repo)).toString();
     QScopedPointer<DVcsJob> job(gitRevList(repo, QStringList(root)));
     bool ret = job->exec();
     Q_ASSERT(ret && job->status()==VcsJob::JobSucceeded && "TODO: provide a fall back in case of failing");
+    Q_UNUSED(ret);
     QStringList commits = job->output().split('\n', QString::SkipEmptyParts);
 //     kDebug() << "\n\n\n commits" << commits << "\n\n\n";
     branchesShas.append(commits);
-    foreach(const QString &branch, branches)
+    foreach(const QString &branch, gitBranches)
     {
         if (branch == root)
             continue;
         QStringList args(branch);
-        foreach(const QString &branch_arg, branches)
+        foreach(const QString &branch_arg, gitBranches)
         {
             if (branch_arg != branch)
                 //man gitRevList for '^'
@@ -804,6 +904,7 @@ void GitPlugin::initBranchHash(const QString &repo)
         QScopedPointer<DVcsJob> job(gitRevList(repo, args));
         bool ret = job->exec();
         Q_ASSERT(ret && job->status()==VcsJob::JobSucceeded && "TODO: provide a fall back in case of failing");
+        Q_UNUSED(ret);
         QStringList commits = job->output().split('\n', QString::SkipEmptyParts);
 //         kDebug() << "\n\n\n commits" << commits << "\n\n\n";
         branchesShas.append(commits);
@@ -847,18 +948,29 @@ void GitPlugin::parseLogOutput(const DVcsJob * job, QList<DVcsEvent>& commits) c
 
 void GitPlugin::parseGitLogOutput(DVcsJob * job)
 {
-    QList<QVariant> commits;
     static QRegExp commitRegex( "^commit (\\w{8})\\w{32}" );
     static QRegExp infoRegex( "^(\\w+):(.*)" );
+    static QRegExp modificationsRegex("(\\d+)\\s+(\\d+)\\s+(.+)"); //18      2       shell/workingsets/workingsetwidget.cpp
+
+    QList<QVariant> commits;
 
     QString contents = job->output();
+    // check if git-log returned anything
+    if (contents.isEmpty()) {
+        job->setResults(commits); // empty list
+        return;
+    }
+
+    // start parsing the output
     QTextStream s(&contents);
 
     VcsEvent item;
     QString message;
     bool pushCommit = false;
+    
     while (!s.atEnd()) {
         QString line = s.readLine();
+        
         if (commitRegex.exactMatch(line)) {
             if (pushCommit) {
                 item.setMessage(message.trimmed());
@@ -877,6 +989,20 @@ void GitPlugin::parseGitLogOutput(DVcsJob * job)
             } else if (cap1 == "Date") {
                 item.setDate(QDateTime::fromTime_t(infoRegex.cap(2).trimmed().split(' ')[0].toUInt()));
             }
+        } else if (modificationsRegex.exactMatch(line)) {
+            int additions = modificationsRegex.cap(1).toUInt();
+            int removals = modificationsRegex.cap(2).toUInt();
+            QString filename = modificationsRegex.cap(3);
+            
+            VcsItemEvent::Actions a;
+            if(additions>0 || removals>0)
+                a=VcsItemEvent::Modified;
+            
+            VcsItemEvent itemEvent;
+            itemEvent.setActions(a);
+            itemEvent.setRepositoryLocation(filename);
+            
+            item.addItem(itemEvent);
         } else if (line.startsWith("    ")) {
             message += line.remove(0, 4);
             message += '\n';
@@ -944,28 +1070,38 @@ void GitPlugin::parseGitStatusOutput_old(DVcsJob* job)
 void GitPlugin::parseGitStatusOutput(DVcsJob* job)
 {
     QStringList outputLines = job->output().split('\n', QString::SkipEmptyParts);
-    const KUrl workingDir = job->directory().absolutePath();
-    const KUrl dotGit = dotGitDirectory(workingDir).absolutePath();
+    KUrl workingDir = job->directory().absolutePath();
+    KUrl dotGit = dotGitDirectory(workingDir).absolutePath();;
+    dotGit.adjustPath(KUrl::AddTrailingSlash);
+    workingDir.adjustPath(KUrl::AddTrailingSlash);
     
     QVariantList statuses;
-    QList<KUrl> processedFiles;
+    KUrl::List processedFiles;
     
     foreach(const QString& line, outputLines) {
         //every line is 2 chars for the status, 1 space then the file desc
         QString curr=line.right(line.size()-3);
         QString state = line.left(2);
         
-        int arrow = curr.indexOf("-> ");
-        if(arrow>=0)
-            curr = curr.right(curr.size()-arrow-3);
+        int arrow = curr.indexOf(" -> ");
+        if(arrow>=0) {
+            VcsStatusInfo status;
+            status.setUrl(KUrl(dotGit, curr.left(arrow)));
+            status.setState(VcsStatusInfo::ItemDeleted);
+            statuses.append(qVariantFromValue<VcsStatusInfo>(status));
+            processedFiles += status.url();
+            
+            curr = curr.mid(arrow+4);
+        }
         
-        KUrl fileUrl = dotGit;
-        fileUrl.addPath(curr);
-        processedFiles.append(fileUrl);
+        if(curr.startsWith('\"') && curr.endsWith('\"')) { //if the path is quoted, unquote
+            curr = curr.mid(1, curr.size()-2);
+        }
         
         VcsStatusInfo status;
-        status.setUrl(fileUrl);
+        status.setUrl(KUrl(dotGit, curr));
         status.setState(messageToState(state));
+        processedFiles.append(status.url());
         
         kDebug() << "Checking git status for " << line << curr << messageToState(state);
         
@@ -980,8 +1116,7 @@ void GitPlugin::parseGitStatusOutput(DVcsJob* job)
     //here we add the already up to date files
     QStringList files = getLsFiles(job->directory(), QStringList() << "-c" << "--" << paths, OutputJob::Silent);
     foreach(const QString& file, files) {
-        KUrl fileUrl = workingDir;
-        fileUrl.addPath(file);
+        KUrl fileUrl(workingDir, file);
         
         if(!processedFiles.contains(fileUrl)) {
             VcsStatusInfo status;
@@ -997,10 +1132,14 @@ void GitPlugin::parseGitStatusOutput(DVcsJob* job)
 void GitPlugin::parseGitVersionOutput(DVcsJob* job)
 {
     QStringList versionString = job->output().trimmed().split(' ').last().split('.');
-    static QList<int> minimumVersion = QList<int>() << 1 << 7;
-    
+    static const QList<int> minimumVersion = QList<int>() << 1 << 7;
     kDebug() << "checking git version" << versionString << "against" << minimumVersion;
     m_oldVersion = false;
+    if (versionString.size() < minimumVersion.size()) {
+        m_oldVersion = true;
+        kWarning() << "invalid git version string:" << job->output().trimmed();
+        return;
+    }
     foreach(int num, minimumVersion) {
         QString curr = versionString.takeFirst();
         int valcurr = curr.toInt();
@@ -1028,9 +1167,9 @@ DVcsJob* GitPlugin::gitRevParse(const QString &repository, const QStringList &ar
     return job;
 }
 
-DVcsJob* GitPlugin::gitRevList(const QString &repository, const QStringList &args)
+DVcsJob* GitPlugin::gitRevList(const QString& directory, const QStringList& args)
 {
-    DVcsJob* job = new DVcsJob(QDir(repository), this, KDevelop::OutputJob::Silent);
+    DVcsJob* job = new DVcsJob(urlDir(directory), this, KDevelop::OutputJob::Silent);
     {
         *job << "git" << "rev-list" << args;
         return job;
@@ -1130,7 +1269,7 @@ VcsJob* GitPlugin::pull(const KDevelop::VcsLocation& localOrRepoLocationSrc, con
 {
     DVcsJob* job = new DVcsJob(urlDir(localRepositoryLocation), this);
     job->setCommunicationMode(KProcess::MergedChannels);
-    *job << "git" << "pull";
+    *job << "git" << "-c" << "color.diff=false" << "pull";
     if(!localOrRepoLocationSrc.localUrl().isEmpty())
         *job << localOrRepoLocationSrc.localUrl().url();
     return job;
@@ -1166,12 +1305,31 @@ VcsJob* GitPlugin::update(const KUrl::List& localLocations, const KDevelop::VcsR
     }
 }
 
+void GitPlugin::setupCommitMessageEditor(const KUrl& localLocation, KTextEdit* editor) const
+{
+    new GitMessageHighlighter(editor);
+    QFile mergeMsgFile(dotGitDirectory(localLocation).filePath(".git/MERGE_MSG"));
+    // Some limit on the file size should be set since whole content is going to be read into
+    // the memory. 1Mb seems to be good value since it's rather strange to have so huge commit
+    // message.
+    static const qint64 maxMergeMsgFileSize = 1024*1024;
+    if (mergeMsgFile.size() > maxMergeMsgFileSize || !mergeMsgFile.open(QIODevice::ReadOnly))
+        return;
+    
+    QString mergeMsg = QString::fromLocal8Bit(mergeMsgFile.read(maxMergeMsgFileSize));
+    editor->setPlainText(mergeMsg);
+}
+
 class GitVcsLocationWidget : public KDevelop::StandardVcsLocationWidget
 {
     public:
         GitVcsLocationWidget(QWidget* parent = 0, Qt::WindowFlags f = 0)
             : StandardVcsLocationWidget(parent, f) {}
-        virtual bool isCorrect() const { return true; }
+
+        virtual bool isCorrect() const
+        {
+            return !url().isEmpty();
+        }
 };
 
 KDevelop::VcsLocationWidget* GitPlugin::vcsLocation(QWidget* parent) const
