@@ -12,32 +12,39 @@
 ***************************************************************************/
 
 #include "localpatchsource.h"
+
 #include <ktemporaryfile.h>
 #include <kdebug.h>
+#include <klocale.h>
 #include <kprocess.h>
+#include <KIcon>
+#include <kshell.h>
 
-QStringList splitArgs( const QString& str ) {
-    QStringList ret;
-    QString current = str;
-    int pos = 0;
-    while ( ( pos = current.indexOf( ' ', pos ) ) != -1 ) {
-        if ( current[ 0 ] == '"' ) {
-            int end = current.indexOf( '"' );
-            if ( end > pos )
-                pos = end;
-        }
-        QString s = current.left( pos );
-        if ( s.length() > 0 )
-            ret << s;
-        current = current.mid( pos + 1 );
-        pos = 0;
-    }
-    if ( current.length() )
-        ret << current;
-    return ret;
+LocalPatchSource::LocalPatchSource()
+    : m_applied(false)
+    , m_depth(0)
+{
 }
 
-void LocalPatchSource::update() {
+LocalPatchSource::~LocalPatchSource()
+{
+    if ( !m_command.isEmpty() && !m_filename.isEmpty() ) {
+        QFile::remove( m_filename.toLocalFile() );
+    }
+}
+
+QString LocalPatchSource::name() const
+{
+    return i18n( "Custom Patch" );
+}
+
+QIcon LocalPatchSource::icon() const
+{
+    return KIcon("text-x-patch");
+}
+
+void LocalPatchSource::update()
+{
     if( !m_command.isEmpty() ) {
         KTemporaryFile temp;
         temp.setSuffix( ".diff" );
@@ -51,7 +58,7 @@ void LocalPatchSource::update() {
             proc.setOutputChannelMode( KProcess::OnlyStdoutChannel );
             proc.setStandardOutputFile( filename );
             ///Try to apply, if it works, the patch is not applied
-            proc << splitArgs( m_command );
+            proc << KShell::splitArgs( m_command );
 
             kDebug() << "calling " << m_command;
 
@@ -71,11 +78,5 @@ void LocalPatchSource::update() {
     }
 }
 
-LocalPatchSource::~LocalPatchSource() {
-    if ( !m_filename.isEmpty() ) {
-        QFile::remove( m_filename.toLocalFile() );
-    }
-}
-
 #include "localpatchsource.moc"
 
