@@ -26,6 +26,7 @@
 #include <QtGui/QHBoxLayout>
 #include <QtGui/QStandardItem>
 #include <QtGui/QToolButton>
+#include <QtGui/QKeyEvent>
 
 #include <kxmlguiwindow.h>
 #include <kiconloader.h>
@@ -66,8 +67,9 @@ using namespace KDevelop;
 
 //BEGIN ProjectManagerFilterAction
 
-ProjectManagerFilterAction::ProjectManagerFilterAction( QObject* parent )
+ProjectManagerFilterAction::ProjectManagerFilterAction(ProjectManagerView* parent)
     : KAction( parent )
+    , m_projectManagerView(parent)
 {
     setIcon(KIcon("view-filter"));
     setText(i18n("Filter..."));
@@ -81,13 +83,29 @@ QWidget* ProjectManagerFilterAction::createWidget( QWidget* parent )
     edit->setClickMessage(i18n("Filter..."));
     edit->setClearButtonShown(true);
     connect(edit, SIGNAL(textChanged(QString)), this, SIGNAL(filterChanged(QString)));
+
+    const QString filterString = m_projectManagerView->filterString();
+    edit->setText(filterString);
+
     return edit;
 }
 
 //END ProjectManagerFilterAction
 
+ProjectManagerViewItemContext::ProjectManagerViewItemContext(const QList< ProjectBaseItem* >& items, ProjectManagerView* view)
+    : ProjectItemContext(items), m_view(view)
+{
+}
+
+ProjectManagerView *ProjectManagerViewItemContext::view() const
+{
+    return m_view;
+}
+
+
 static const char* sessionConfigGroup = "ProjectManagerView";
 static const char* splitterStateConfigKey = "splitterState";
+static const char* filterConfigKey = "filter";
 static const int projectTreeViewStrechFactor = 75; // %
 static const int projectBuildSetStrechFactor = 25; // %
 
@@ -96,6 +114,8 @@ ProjectManagerView::ProjectManagerView( ProjectManagerViewPlugin* plugin, QWidge
 {
     m_ui->setupUi( this );
 
+    m_ui->projectTreeView->installEventFilter(this);
+
     setWindowIcon( SmallIcon( "project-development" ) );
 
     KConfigGroup pmviewConfig(ICore::self()->activeSession()->config(), sessionConfigGroup);
@@ -122,12 +142,7 @@ ProjectManagerView::ProjectManagerView( ProjectManagerViewPlugin* plugin, QWidge
     addAction(plugin->actionCollection()->action("project_install"));
     addAction(plugin->actionCollection()->action("project_clean"));
 
-    connect(m_ui->projectTreeView, SIGNAL(activateUrl(const KUrl&)), this, SLOT(openUrl(const KUrl&)));
-
-//     m_filters = new KLineEdit(this);
-//     m_filters->setClearButtonShown(true);
-//     connect(d->m_filters, SIGNAL(returnPressed()), this, SLOT(filtersChanged()));
-//     vbox->addWidget( m_filters );
+    connect(m_ui->projectTreeView, SIGNAL(activateUrl(KUrl)), this, SLOT(openUrl(KUrl)));
 
     m_ui->buildSetView->setProjectView( this );
 
@@ -137,16 +152,22 @@ ProjectManagerView::ProjectManagerView( ProjectManagerViewPlugin* plugin, QWidge
 
     m_ui->projectTreeView->setModel( m_modelFilter );
 
+    QString filterText;
+    if (pmviewConfig.hasKey(filterConfigKey)) {
+        filterText = pmviewConfig.readEntry(filterConfigKey, QString());
+    }
+    setFilterString(filterText);
+
     ProjectManagerFilterAction* filterAction = new ProjectManagerFilterAction(this);
     connect(filterAction, SIGNAL(filterChanged(QString)),
-            this, SLOT(filterChanged(QString)));
+            this, SLOT(setFilterString(QString)));
     addAction(filterAction);
 
-    connect( m_ui->projectTreeView->selectionModel(), SIGNAL(selectionChanged( const QItemSelection&, const QItemSelection&) ),
-             this, SLOT(selectionChanged() ) );
-    connect( KDevelop::ICore::self()->documentController(), SIGNAL(documentClosed(KDevelop::IDocument*) ),
+    connect( m_ui->projectTreeView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
+             this, SLOT(selectionChanged()) );
+    connect( KDevelop::ICore::self()->documentController(), SIGNAL(documentClosed(KDevelop::IDocument*)),
              SLOT(updateSyncAction()));
-    connect( KDevelop::ICore::self()->documentController(), SIGNAL(documentActivated(KDevelop::IDocument*) ),
+    connect( KDevelop::ICore::self()->documentController(), SIGNAL(documentActivated(KDevelop::IDocument*)),
              SLOT(updateSyncAction()));
     connect( qobject_cast<Sublime::MainWindow*>(KDevelop::ICore::self()->uiController()->activeMainWindow()), SIGNAL(areaChanged(Sublime::Area*)),
              SLOT(updateSyncAction()));
@@ -160,6 +181,29 @@ ProjectManagerView::ProjectManagerView( ProjectManagerViewPlugin* plugin, QWidge
     m_ui->projectTreeView->header()->setResizeMode( QHeaderView::ResizeToContents );
 }
 
+bool ProjectManagerView::eventFilter(QObject* obj, QEvent* event)
+{
+    if (obj == m_ui->projectTreeView) {
+        if (event->type() == QEvent::KeyRelease) {
+            QKeyEvent* keyEvent = static_cast<QKeyEvent*>(event);
+            if (keyEvent->key() == Qt::Key_Delete) {
+                m_plugin->removeItems(selectedItems());
+                return true;
+            } else if (keyEvent->key() == Qt::Key_F2) {
+                m_plugin->renameItems(selectedItems());
+                return true;
+            } else if (keyEvent->key() == Qt::Key_C && keyEvent->modifiers() == Qt::ControlModifier) {
+                m_plugin->copyFromContextMenu();
+                return true;
+            } else if (keyEvent->key() == Qt::Key_V && keyEvent->modifiers() == Qt::ControlModifier) {
+                m_plugin->pasteFromContextMenu();
+                return true;
+            }
+        }
+    }
+    return QObject::eventFilter(obj, event);
+}
+
 void ProjectManagerView::selectionChanged()
 {
     m_ui->buildSetView->selectionChanged();
@@ -168,7 +212,8 @@ void ProjectManagerView::selectionChanged()
     {
         selected << m_modelFilter->itemFromProxyIndex( idx );
     }
-    KDevelop::ICore::self()->selectionController()->updateSelection( new ProjectItemContext( selected ) );
+    selected.removeAll(0);
+    KDevelop::ICore::self()->selectionController()->updateSelection( new ProjectManagerViewItemContext( selected, this ) );
 }
 
 void ProjectManagerView::updateSyncAction()
@@ -180,6 +225,7 @@ ProjectManagerView::~ProjectManagerView()
 {
     KConfigGroup pmviewConfig(ICore::self()->activeSession()->config(), sessionConfigGroup);
     pmviewConfig.writeEntry(splitterStateConfigKey, m_ui->splitter->saveState());
+    pmviewConfig.writeEntry(filterConfigKey, m_filterString);
     pmviewConfig.sync();
 }
 
@@ -198,6 +244,22 @@ QList<KDevelop::ProjectBaseItem*> ProjectManagerView::selectedItems() const
     return items;
 }
 
+void ProjectManagerView::selectItems(const QList< ProjectBaseItem* >& items)
+{
+    QItemSelection selection;
+    foreach (ProjectBaseItem *item, items) {
+        QModelIndex indx = m_modelFilter->mapFromSource(item->model()->indexFromItem(item));
+        selection.append(QItemSelectionRange(indx, indx));
+        m_ui->projectTreeView->setCurrentIndex(indx);
+    }
+    m_ui->projectTreeView->selectionModel()->select(selection, QItemSelectionModel::ClearAndSelect);
+}
+
+void ProjectManagerView::expandItem(ProjectBaseItem* item)
+{
+    m_ui->projectTreeView->expand( m_modelFilter->mapFromSource(item->model()->indexFromItem(item)));
+}
+
 void ProjectManagerView::locateCurrentDocument()
 {
     ICore::self()->uiController()->raiseToolView(this);
@@ -242,8 +304,14 @@ void ProjectManagerView::openUrl( const KUrl& url )
     IOpenWith::openFiles(KUrl::List() << url);
 }
 
-void ProjectManagerView::filterChanged(const QString &text)
+QString ProjectManagerView::filterString() const
 {
+    return m_filterString;
+}
+
+void ProjectManagerView::setFilterString(const QString &text)
+{
+    m_filterString = text;
     m_modelFilter->setFilterString(text);
 }
 
