@@ -20,8 +20,10 @@
 #include "projectmanagerviewplugin.h"
 
 #include <QtCore/QList>
+#include <QMimeData>
 #include <QtGui/QInputDialog>
 #include <QtGui/QApplication>
+#include <QClipboard>
 
 #include <kaction.h>
 #include <kactioncollection.h>
@@ -47,7 +49,6 @@
 #include <interfaces/iselectioncontroller.h>
 
 #include "projectmanagerview.h"
-#include "builditembuilderjob.h"
 
 using namespace KDevelop;
 
@@ -107,28 +108,34 @@ ProjectManagerViewPlugin::ProjectManagerViewPlugin( QObject *parent, const QVari
     d->m_buildAll->setIcon(KIcon("run-build"));
     connect( d->m_buildAll, SIGNAL(triggered()), this, SLOT(buildAllProjects()) );
     actionCollection()->addAction( "project_buildall", d->m_buildAll );
+
     d->m_build = new KAction( i18n("Build Selection"), this );
+    d->m_build->setIconText( i18n("Build") );
     d->m_build->setShortcut( Qt::Key_F8 );
     d->m_build->setIcon(KIcon("run-build"));
     d->m_build->setEnabled( false );
     connect( d->m_build, SIGNAL(triggered()), this, SLOT(buildProjectItems()) );
     actionCollection()->addAction( "project_build", d->m_build );
     d->m_install = new KAction( i18n("Install Selection"), this );
+    d->m_install->setIconText( i18n("Install") );
     d->m_install->setIcon(KIcon("run-build-install"));
     d->m_install->setEnabled( false );
     connect( d->m_install, SIGNAL(triggered()), this, SLOT(installProjectItems()) );
     actionCollection()->addAction( "project_install", d->m_install );
     d->m_clean = new KAction( i18n("Clean Selection"), this );
+    d->m_clean->setIconText( i18n("Clean") );
     d->m_clean->setIcon(KIcon("run-build-clean"));
     d->m_clean->setEnabled( false );
     connect( d->m_clean, SIGNAL(triggered()), this, SLOT(cleanProjectItems()) );
     actionCollection()->addAction( "project_clean", d->m_clean );
     d->m_configure = new KAction( i18n("Configure Selection"), this );
+    d->m_configure->setIconText( i18n("Configure") );
     d->m_configure->setIcon(KIcon("run-build-configure"));
     d->m_configure->setEnabled( false );
     connect( d->m_configure, SIGNAL(triggered()), this, SLOT(configureProjectItems()) );
     actionCollection()->addAction( "project_configure", d->m_configure );
     d->m_prune = new KAction( i18n("Prune Selection"), this );
+    d->m_prune->setIconText( i18n("Prune") );
     d->m_prune->setIcon(KIcon("run-build-prune"));
     d->m_prune->setEnabled( false );
     connect( d->m_prune, SIGNAL(triggered()), this, SLOT(pruneProjectItems()) );
@@ -201,11 +208,13 @@ ContextMenuExtension ProjectManagerViewPlugin::contextMenuExtension( KDevelop::C
     bool needsFolderItems = true;
     bool needsRemoveAndRename = true;
     bool needsRemoveTargetFiles = true;
+    bool needsPaste = true;
 
     //needsCreateFile if there is one item and it's a folder or target
     needsCreateFile &= (items.count() == 1) && (items.first()->folder() || items.first()->target());
     //needsCreateFolder if there is one item and it's a folder
     needsCreateFolder &= (items.count() == 1) && (items.first()->folder());
+    needsPaste = needsCreateFolder;
     
     foreach( ProjectBaseItem* item, items ) {
         d->ctxProjectItemList << item;
@@ -237,16 +246,17 @@ ContextMenuExtension ProjectManagerViewPlugin::contextMenuExtension( KDevelop::C
         connect( action, SIGNAL(triggered()), this, SLOT(createFolderFromContextMenu()) );
         menuExt.addAction( ContextMenuExtension::FileGroup, action );
     }
+
     if ( needsBuildItems ) {
-        KAction* action = new KAction( i18n( "Build" ), this );
+        KAction* action = new KAction( i18nc( "@action", "Build" ), this );
         action->setIcon(KIcon("run-build"));
         connect( action, SIGNAL(triggered()), this, SLOT(buildItemsFromContextMenu()) );
         menuExt.addAction( ContextMenuExtension::BuildGroup, action );
-        action = new KAction( i18n( "Install" ), this );
+        action = new KAction( i18nc( "@action", "Install" ), this );
         action->setIcon(KIcon("run-install"));
         connect( action, SIGNAL(triggered()), this, SLOT(installItemsFromContextMenu()) );
         menuExt.addAction( ContextMenuExtension::BuildGroup, action );
-        action = new KAction( i18n( "Clean" ), this );
+        action = new KAction( i18nc( "@action", "Clean" ), this );
         action->setIcon(KIcon("run-clean"));
         connect( action, SIGNAL(triggered()), this, SLOT(cleanItemsFromContextMenu()) );
         menuExt.addAction( ContextMenuExtension::BuildGroup, action );
@@ -254,6 +264,7 @@ ContextMenuExtension ProjectManagerViewPlugin::contextMenuExtension( KDevelop::C
         connect( action, SIGNAL(triggered()), this, SLOT(addItemsFromContextMenuToBuildset()) );
         menuExt.addAction( ContextMenuExtension::BuildGroup, action );
     }
+
     if ( needsCloseProjects ) {
         KAction* close = new KAction( i18np( "Close Project", "Close Projects", items.count() ), this );
         close->setIcon(KIcon("project-development-close"));
@@ -282,26 +293,19 @@ ContextMenuExtension ProjectManagerViewPlugin::contextMenuExtension( KDevelop::C
         connect( remove, SIGNAL(triggered()), this, SLOT(removeTargetFilesFromContextMenu()) );
         menuExt.addAction( ContextMenuExtension::FileGroup, remove );
     }
-    return menuExt;
-}
 
-
-KDevelop::IProjectBuilder* ProjectManagerViewPlugin::getProjectBuilder( KDevelop::ProjectBaseItem* item )
-{
-    if( !item )
-        return 0;
-    IProject* project = item->project();
-    if (!project)
-        return 0;
-
-    ProjectFolderItem* prjitem = project->projectItem();
-    IPlugin* fmgr = project->managerPlugin();
-    IBuildSystemManager* mgr = fmgr->extension<IBuildSystemManager>();
-    if( mgr )
     {
-        return mgr->builder( prjitem );
+        KAction* copy = KStandardAction::copy(this, SLOT(copyFromContextMenu()), this);
+        copy->setShortcutContext(Qt::WidgetShortcut);
+        menuExt.addAction( ContextMenuExtension::FileGroup, copy );
     }
-    return 0;
+    if (needsPaste) {
+        KAction* paste = KStandardAction::paste(this, SLOT(pasteFromContextMenu()), this);
+        paste->setShortcutContext(Qt::WidgetShortcut);
+        menuExt.addAction( ContextMenuExtension::FileGroup, paste );
+    }
+
+    return menuExt;
 }
 
 void ProjectManagerViewPlugin::closeProjects()
@@ -324,68 +328,91 @@ void ProjectManagerViewPlugin::closeProjects()
 
 void ProjectManagerViewPlugin::installItemsFromContextMenu()
 {
-     ICore::self()->runController()->registerJob( new BuildItemBuilderJob( KDevelop::BuilderJob::Install, d->ctxProjectItemList ) );
+    runBuilderJob( BuilderJob::Install, d->ctxProjectItemList );
     d->ctxProjectItemList.clear();
 }
 
 void ProjectManagerViewPlugin::cleanItemsFromContextMenu()
 {
-     ICore::self()->runController()->registerJob( new BuildItemBuilderJob( KDevelop::BuilderJob::Clean, d->ctxProjectItemList ) );
+    runBuilderJob( BuilderJob::Clean, d->ctxProjectItemList );
     d->ctxProjectItemList.clear();
 }
 
 void ProjectManagerViewPlugin::buildItemsFromContextMenu()
 {
-     ICore::self()->runController()->registerJob( new BuildItemBuilderJob( KDevelop::BuilderJob::Build, d->ctxProjectItemList ) );
+    runBuilderJob( BuilderJob::Build, d->ctxProjectItemList );
     d->ctxProjectItemList.clear();
 }
 
-void ProjectManagerViewPlugin::buildAllProjects()
+QList<ProjectBaseItem*> ProjectManagerViewPlugin::collectAllProjects()
 {
     QList<KDevelop::ProjectBaseItem*> items;
     foreach( KDevelop::IProject* project, core()->projectController()->projects() )
     {
         items << project->projectItem();
     }
-    ICore::self()->runController()->registerJob( new BuildItemBuilderJob( KDevelop::BuilderJob::Build, items ) );
+    return items;
 }
 
-void ProjectManagerViewPlugin::runBuilderJob( KDevelop::BuilderJob::BuildType t )
+void ProjectManagerViewPlugin::buildAllProjects()
+{
+    runBuilderJob( BuilderJob::Build, collectAllProjects() );
+}
+
+QList<ProjectBaseItem*> ProjectManagerViewPlugin::collectItems()
 {
     QList<ProjectBaseItem*> items;
-    if( !ICore::self()->projectController()->buildSetModel()->items().isEmpty() )
+    QList<BuildItem> buildItems = ICore::self()->projectController()->buildSetModel()->items();
+    if( !buildItems.isEmpty() )
     {
-        ICore::self()->runController()->registerJob( new BuildItemBuilderJob( t, ICore::self()->projectController()->buildSetModel()->items() ) );
+        foreach( const BuildItem& buildItem, buildItems )
+        {
+            if( ProjectBaseItem* item = buildItem.findItem() )
+            {
+                items << item;
+            }
+        }
+
     } else
     {
         KDevelop::ProjectItemContext* ctx = dynamic_cast<KDevelop::ProjectItemContext*>(ICore::self()->selectionController()->currentSelection());
-        ICore::self()->runController()->registerJob( new BuildItemBuilderJob( t, ctx->items() ) );
+        items = ctx->items();
     }
+
+    return items;
+}
+
+void ProjectManagerViewPlugin::runBuilderJob( BuilderJob::BuildType type, QList<ProjectBaseItem*> items )
+{
+    BuilderJob* builder = new BuilderJob;
+    builder->addItems( type, items );
+    builder->updateJobName();
+    ICore::self()->runController()->registerJob( builder );
 }
 
 void ProjectManagerViewPlugin::installProjectItems()
 {
-    runBuilderJob( KDevelop::BuilderJob::Install );
+    runBuilderJob( KDevelop::BuilderJob::Install, collectItems() );
 }
 
 void ProjectManagerViewPlugin::pruneProjectItems()
 {
-    runBuilderJob( KDevelop::BuilderJob::Prune );
+    runBuilderJob( KDevelop::BuilderJob::Prune, collectItems() );
 }
 
 void ProjectManagerViewPlugin::configureProjectItems()
 {
-    runBuilderJob( KDevelop::BuilderJob::Configure );
+    runBuilderJob( KDevelop::BuilderJob::Configure, collectItems() );
 }
 
 void ProjectManagerViewPlugin::cleanProjectItems()
 {
-    runBuilderJob( KDevelop::BuilderJob::Clean );
+    runBuilderJob( KDevelop::BuilderJob::Clean, collectItems() );
 }
 
 void ProjectManagerViewPlugin::buildProjectItems()
 {
-    runBuilderJob( KDevelop::BuilderJob::Build );
+    runBuilderJob( KDevelop::BuilderJob::Build, collectItems() );
 }
 
 void ProjectManagerViewPlugin::addItemsFromContextMenuToBuildset( )
@@ -464,21 +491,27 @@ void ProjectManagerViewPlugin::createFolderFromContextMenu( )
 
 void ProjectManagerViewPlugin::removeFromContextMenu()
 {
+    removeItems(d->ctxProjectItemList);
+}
+
+void ProjectManagerViewPlugin::removeItems(const QList< ProjectBaseItem* >& items)
+{
+    if (items.isEmpty()) {
+        return;
+    }
+
     //copy the list of selected items and sort it to guarantee parents will come before children
-    QMap< IProjectFileManager*, QList<KDevelop::ProjectBaseItem*> > filteredItems;
-    filteredItems[0] += d->ctxProjectItemList;
-    qSort(filteredItems[0].begin(), filteredItems[0].end(), ProjectBaseItem::urlLessThan);
+    QList<KDevelop::ProjectBaseItem*> sortedItems = items;
+    qSort(sortedItems.begin(), sortedItems.end(), ProjectBaseItem::urlLessThan);
 
     KUrl lastFolder;
+    QMap< IProjectFileManager*, QList<KDevelop::ProjectBaseItem*> > filteredItems;
     QStringList itemPaths;
-
-    foreach( KDevelop::ProjectBaseItem* item, filteredItems[0] )
+    foreach( KDevelop::ProjectBaseItem* item, sortedItems )
     {
-        Q_ASSERT(item->folder() || item->file());
-        Q_ASSERT(!item->file() || !item->file()->parent()->target());
-
-        if (item->folder() || item->file())
-        {
+        if (item->isProjectRoot()) {
+            continue;
+        } else if (item->folder() || item->file()) {
             //make sure no children of folders that will be deleted are listed
             if (lastFolder.isParentOf(item->url())) {
                 continue;
@@ -486,11 +519,18 @@ void ProjectManagerViewPlugin::removeFromContextMenu()
                 lastFolder = item->url();
             }
 
-            filteredItems[item->project()->projectFileManager()] << item;
-            itemPaths << item->url().path();
+            IProjectFileManager* manager = item->project()->projectFileManager();
+            if (manager) {
+                filteredItems[manager] << item;
+                itemPaths << item->url().path();
+            }
         }
     }
 
+    if (filteredItems.isEmpty()) {
+        return;
+    }
+
     if (KMessageBox::warningYesNoList(
             QApplication::activeWindow(),
             i18np("Do you really want to delete this item?",
@@ -506,8 +546,8 @@ void ProjectManagerViewPlugin::removeFromContextMenu()
     QMap< IProjectFileManager*, QList<KDevelop::ProjectBaseItem*> >::iterator it;
     for (it = filteredItems.begin(); it != filteredItems.end(); ++it)
     {
-        if (it.key())
-            it.key()->removeFilesAndFolders(it.value());
+        Q_ASSERT(it.key());
+        it.key()->removeFilesAndFolders(it.value());
     }
 }
 
@@ -525,9 +565,18 @@ void ProjectManagerViewPlugin::removeTargetFilesFromContextMenu()
 
 void ProjectManagerViewPlugin::renameItemFromContextMenu()
 {
-    QWidget* window=ICore::self()->uiController()->activeMainWindow()->window();
+    renameItems(d->ctxProjectItemList);
+}
 
-    foreach( KDevelop::ProjectBaseItem* item, d->ctxProjectItemList )
+void ProjectManagerViewPlugin::renameItems(const QList< ProjectBaseItem* >& items)
+{
+    if (items.isEmpty()) {
+        return;
+    }
+
+    QWidget* window = ICore::self()->uiController()->activeMainWindow()->window();
+
+    foreach( KDevelop::ProjectBaseItem* item, items )
     {
         if ((item->type()!=ProjectBaseItem::BuildFolder
                 && item->type()!=ProjectBaseItem::Folder
@@ -547,8 +596,7 @@ void ProjectManagerViewPlugin::renameItemFromContextMenu()
 
         if (!name.isEmpty() && name != src) {
             ProjectBaseItem::RenameStatus status = item->rename( name );
-            
-            QWidget* window(QApplication::activeWindow());
+
             switch(status) {
                 case ProjectBaseItem::RenameOk:
                     break;
@@ -600,5 +648,63 @@ void ProjectManagerViewPlugin::createFileFromContextMenu( )
     }
 }
 
+void ProjectManagerViewPlugin::copyFromContextMenu()
+{
+    KDevelop::ProjectItemContext* ctx = dynamic_cast<KDevelop::ProjectItemContext*>(ICore::self()->selectionController()->currentSelection());
+    KUrl::List urls;
+    foreach (ProjectBaseItem* item, ctx->items()) {
+        if (item->folder() || item->file()) {
+            urls << item->url();
+        }
+    }
+    kDebug() << urls;
+    if (!urls.isEmpty()) {
+        QMimeData *data = new QMimeData;
+        urls.populateMimeData(data);
+        qApp->clipboard()->setMimeData(data);
+    }
+}
+
+void ProjectManagerViewPlugin::pasteFromContextMenu()
+{
+    KDevelop::ProjectItemContext* ctx = dynamic_cast<KDevelop::ProjectItemContext*>(ICore::self()->selectionController()->currentSelection());
+    if (ctx->items().count() != 1)
+        return; //do nothing if multiple or none items are selected
+
+    ProjectBaseItem* destItem = ctx->items().first();
+    if (!destItem->folder())
+        return; //do nothing if the target is not a directory
+
+    const QMimeData* data = qApp->clipboard()->mimeData();
+    kDebug() << data->urls();
+    KUrl::List urls(data->urls());
+    bool success = destItem->project()->projectFileManager()->copyFilesAndFolders(urls, destItem->folder());
+
+    if (success) {
+        ProjectManagerViewItemContext* viewCtx = dynamic_cast<ProjectManagerViewItemContext*>(ICore::self()->selectionController()->currentSelection());
+        if (viewCtx) {
+
+            //expand target folder
+            viewCtx->view()->expandItem(destItem);
+
+            //and select new items
+            QList<ProjectBaseItem*> newItems;
+            foreach (const KUrl &url, urls) {
+                KUrl targetUrl = destItem->url();
+                targetUrl.addPath(url.fileName());
+                foreach (ProjectBaseItem *item, destItem->children()) {
+                    KUrl itemUrl = item->url();
+                    itemUrl.adjustPath(KUrl::RemoveTrailingSlash); //required to correctly compare urls
+                    if (itemUrl == targetUrl) {
+                        newItems << item;
+                    }
+                }
+            }
+            viewCtx->view()->selectItems(newItems);
+        }
+    }
+}
+
+
 #include "projectmanagerviewplugin.moc"
 
