@@ -137,6 +137,8 @@ void QuickOpenModel::enableProviders( const QStringList& _items, const QStringLi
 
 void QuickOpenModel::textChanged( const QString& str )
 {
+  if( m_filterText == str )
+      return;
   m_filterText = str;
   foreach( const ProviderEntry& provider, m_providers )
     if( provider.enabled )
@@ -230,6 +232,16 @@ int QuickOpenModel::rowCount( const QModelIndex& i ) const
   return count;
 }
 
+int QuickOpenModel::unfilteredRowCount() const
+{
+  int count = 0;
+  foreach( const ProviderEntry& provider, m_providers )
+    if( provider.enabled )
+      count += provider.provider->unfilteredItemCount();
+
+  return count;
+}
+
 int QuickOpenModel::columnCount() const
 {
   return 2;
@@ -345,24 +357,18 @@ QuickOpenDataPointer QuickOpenModel::getItem( int row, bool noReset ) const {
     uint itemCount = provider.provider->itemCount();
     if( (uint)row < itemCount )
     {
-      QList<QuickOpenDataPointer> items = provider.provider->data( row, row+1 );
+      QuickOpenDataPointer item = provider.provider->data( row );
 
       if(!noReset && provider.provider->itemCount() != itemCount) {
           kDebug() << "item-count in provider has changed, resetting model";
           m_resetTimer->start(0);
           m_resetBehindRow = rowOffset + row; //Don't reset everything, only everything behind this position
       }
-      
-      if( items.isEmpty() )
-      {
-//         kWarning() << "Provider returned no item";
-        return QuickOpenDataPointer();
-      } else {
+
 #ifdef QUICKOPEN_USE_ITEM_CACHING
-        m_cachedData[row+rowOffset] = items.first();
+      m_cachedData[row+rowOffset] = item;
 #endif
-        return items.first();
-      }
+      return item;
     } else {
       row -= provider.provider->itemCount();
       rowOffset += provider.provider->itemCount();
@@ -388,7 +394,6 @@ QSet<IndexedString> QuickOpenModel::fileSet() const {
   return merged;
 }
 
-
 QTreeView* QuickOpenModel::treeView() const {
   return m_treeView;
 }
