@@ -22,10 +22,13 @@
 #include <QtGui/QSortFilterProxyModel>
 #include <QtCore/QThread>
 #include <qtest_kde.h>
+#include <KMimeType>
 
 #include <projectmodel.h>
+#include <projectproxymodel.h>
 #include <tests/modeltest.h>
-#include "dummyproject.h"
+#include <tests/testproject.h>
+#include <projectproxymodel.h>
 #include <tests/kdevsignalspy.h>
 #include <tests/autotestshell.h>
 #include <tests/testcore.h>
@@ -39,6 +42,8 @@ using KDevelop::ProjectLibraryTargetItem;
 using KDevelop::ProjectTargetItem;
 using KDevelop::ProjectBuildFolderItem;
 
+using KDevelop::TestProject;
+
 class AddItemThread : public QThread
 {
 Q_OBJECT
@@ -85,14 +90,26 @@ private:
     ProjectModel* model;
 };
 
+void debugItemModel(QAbstractItemModel* m, const QModelIndex& parent=QModelIndex(), int depth=0)
+{
+    Q_ASSERT(m);
+    qDebug() << QByteArray(depth*2, '-') << m->data(parent).toString();
+    for(int i=0; i<m->rowCount(parent); i++) {
+        debugItemModel(m, m->index(i, 0, parent), depth+1);
+    }
+}
+
 void ProjectModelTest::initTestCase()
 {
     KDevelop::AutoTestShell::init();
     KDevelop::TestCore::initialize(KDevelop::Core::NoUi);
 
     qRegisterMetaType<QModelIndex>("QModelIndex");
-    model = new ProjectModel( this );
-    modelTest = new ModelTest( model, this );
+    model = KDevelop::ICore::self()->projectController()->projectModel();
+    new ModelTest( model, this );
+    proxy = new ProjectProxyModel( model );
+    new ModelTest(proxy, proxy);
+    proxy->setSourceModel(model);
 }
 
 void ProjectModelTest::init()
@@ -102,7 +119,6 @@ void ProjectModelTest::init()
 
 void ProjectModelTest::cleanupTestCase()
 {
-    delete model;
     KDevelop::TestCore::shutdown();
 }
 
@@ -369,10 +385,10 @@ void ProjectModelTest::testRename()
     QFETCH( QString, expectedItemText );
     QFETCH( int, expectedRenameCode );
 
-    DummyProject* proj = new DummyProject( "DummyProject", 0 );
-    ProjectFolderItem* rootItem = new ProjectFolderItem( proj, KUrl("file:///dummyprojectfolder"), 0);
+    const KUrl projectFolder = KUrl("file:///dummyprojectfolder");
+    TestProject* proj = new TestProject;
+    ProjectFolderItem* rootItem = new ProjectFolderItem( proj, projectFolder, 0);
     proj->setProjectItem( rootItem );
-    model->appendRow( rootItem );
 
     new ProjectFileItem(proj, KUrl("existing"), rootItem);
 
@@ -380,11 +396,11 @@ void ProjectModelTest::testRename()
     if( itemType == ProjectBaseItem::Target ) {
         item = new ProjectTargetItem( proj, itemText, rootItem );
     } else if( itemType == ProjectBaseItem::File ) {
-        item = new ProjectFileItem( proj, KUrl(itemText), rootItem );
+        item = new ProjectFileItem( proj, KUrl(projectFolder, itemText), rootItem );
     } else if( itemType == ProjectBaseItem::Folder ) {
-        item = new ProjectFolderItem( proj, KUrl(itemText), rootItem );
+        item = new ProjectFolderItem( proj, KUrl(projectFolder, itemText), rootItem );
     } else if( itemType == ProjectBaseItem::BuildFolder ) {
-        item = new ProjectBuildFolderItem( proj, KUrl(itemText), rootItem );
+        item = new ProjectBuildFolderItem( proj, KUrl(projectFolder, itemText), rootItem );
     }
     Q_ASSERT( item );
 
@@ -425,6 +441,14 @@ void ProjectModelTest::testRename_data()
     << true
     << QString::fromLatin1("otherfile.cpp")
     << (int)ProjectBaseItem::RenameOk;
+
+    QTest::newRow("SourceAndDestinationFileEqual")
+    << (int)ProjectBaseItem::File
+    << QString::fromLatin1("newfile.cpp")
+    << QString::fromLatin1("newfile.cpp")
+    << false
+    << QString::fromLatin1("newfile.cpp")
+    << (int)ProjectBaseItem::RenameOk;
     
     QTest::newRow("RenameableFolder")
     << (int)ProjectBaseItem::Folder
@@ -433,6 +457,14 @@ void ProjectModelTest::testRename_data()
     << true
     << QString::fromLatin1("otherfolder")
     << (int)ProjectBaseItem::RenameOk;
+
+    QTest::newRow("SourceAndDestinationFolderEqual")
+    << (int)ProjectBaseItem::Folder
+    << QString::fromLatin1("newfolder")
+    << QString::fromLatin1("newfolder")
+    << false
+    << QString::fromLatin1("newfolder")
+    << (int)ProjectBaseItem::RenameOk;
     
     QTest::newRow("RenameableBuildFolder")
     << (int)ProjectBaseItem::BuildFolder
@@ -442,6 +474,14 @@ void ProjectModelTest::testRename_data()
     << QString::fromLatin1("otherbfolder")
     << (int)ProjectBaseItem::RenameOk;
 
+    QTest::newRow("SourceAndDestinationBuildFolderEqual")
+    << (int)ProjectBaseItem::BuildFolder
+    << QString::fromLatin1("newbfolder")
+    << QString::fromLatin1("newbfolder")
+    << false
+    << QString::fromLatin1("newbfolder")
+    << (int)ProjectBaseItem::RenameOk;
+
     QTest::newRow("ExistingFileError")
     << (int)ProjectBaseItem::Folder
     << QString::fromLatin1("mynew")
@@ -461,10 +501,9 @@ void ProjectModelTest::testRename_data()
 
 void ProjectModelTest::testWithProject()
 {
-    DummyProject* proj = new DummyProject( "DummyProject", 0 );
+    TestProject* proj = new TestProject();
     ProjectFolderItem* rootItem = new ProjectFolderItem( proj, KUrl("file:///dummyprojectfolder"), 0);
     proj->setProjectItem( rootItem );
-    model->appendRow( rootItem );
     ProjectBaseItem* item = model->itemFromIndex( model->index( 0, 0 ) );
     QCOMPARE( item, rootItem );
     QCOMPARE( item->text(), proj->name() );
@@ -484,22 +523,6 @@ void ProjectModelTest::testAddItemInThread()
     QCOMPARE( qApp->thread(), check.threadOfSignalEmission() );
 }
 
-void ProjectModelTest::testDeleteLots()
-{
-    delete modelTest;
-
-    const int items = 10000;
-    ProjectBaseItem* root = new ProjectBaseItem( 0, "root", 0 );
-    model->appendRow( root );
-    for ( int i = 0; i < items; ++i ) {
-        new ProjectBaseItem( 0, QString::number(i), root );
-    }
-    QCOMPARE(root->children().size(), items);
-    delete root;
-
-    modelTest = new ModelTest( model, this );
-}
-
 void ProjectModelTest::testItemsForUrl()
 {
     QFETCH(KUrl, url);
@@ -524,21 +547,69 @@ void ProjectModelTest::testItemsForUrl_data()
     QTest::addColumn<int>("matches");
 
     {
-        ProjectFolderItem* root = new ProjectFolderItem(0, KUrl("file:////tmp/"));
+        ProjectFolderItem* root = new ProjectFolderItem(0, KUrl("file:///tmp/"));
         ProjectFileItem* file = new ProjectFileItem(0, KUrl(root->url(), "a"), root);
         QTest::newRow("find one") << file->url() << static_cast<ProjectBaseItem*>(root) << 1;
     }
 
     {
-        ProjectFolderItem* root = new ProjectFolderItem(0, KUrl("file:////tmp/"));
+        ProjectFolderItem* root = new ProjectFolderItem(0, KUrl("file:///tmp/"));
         ProjectFolderItem* folder = new ProjectFolderItem(0, KUrl(root->url(), "a"), root);
         ProjectFileItem* file = new ProjectFileItem(0, KUrl(folder->url(), "foo"), folder);
         ProjectTargetItem* target = new ProjectTargetItem(0, "b", root);
         ProjectFileItem* file2 = new ProjectFileItem(0, file->url(), target);
+        Q_UNUSED(file2);
         QTest::newRow("find two") << file->url() << static_cast<ProjectBaseItem*>(root) << 2;
     }
 }
 
+void ProjectModelTest::testProjectProxyModel()
+{
+    ProjectFolderItem* root = new ProjectFolderItem(0, KUrl("file:///tmp/"));
+    new ProjectFileItem(0, KUrl("file:///tmp/a1/"), root);
+    new ProjectFileItem(0, KUrl("file:///tmp/b1/"), root);
+    new ProjectFileItem(0, KUrl("file:///tmp/c1/"), root);
+    new ProjectFileItem(0, KUrl("file:///tmp/d1/"), root);
+    model->appendRow(root);
+    
+    QModelIndex proxyRoot = proxy->mapFromSource(root->index());
+    QCOMPARE(model->rowCount(root->index()), 4);
+    QCOMPARE(proxy->rowCount(proxyRoot), 4);
+    
+    proxy->setFilterString("*1");
+    QCOMPARE(proxy->rowCount(proxyRoot), 4);
+    
+    proxy->setFilterString("a*");
+    debugItemModel(proxy);
+    QCOMPARE(proxy->rowCount(proxyRoot), 1);
+    
+    model->clear();
+}
+
+void ProjectModelTest::testProjectFileSet()
+{
+    TestProject* project = new TestProject;
+
+    QVERIFY(project->fileSet().isEmpty());
+    KUrl url("file:///tmp/a");
+    ProjectFileItem* item = new ProjectFileItem(project, url, project->projectItem());
+    QCOMPARE(project->fileSet().size(), 1);
+    qDebug() << url << project->fileSet().begin()->toUrl();
+    QCOMPARE(project->fileSet().begin()->toUrl(), url);
+    delete item;
+    QVERIFY(project->fileSet().isEmpty());
+}
+
+void ProjectModelTest::testProjectFileIcon()
+{
+    ProjectFileItem* item = new ProjectFileItem(0, KUrl("/tmp/foo.txt"));
+    const QString txtIcon = KMimeType::iconNameForUrl(item->url());
+    QCOMPARE(item->iconName(), txtIcon);
+    item->setUrl(KUrl("/tmp/bar.cpp"));
+    QCOMPARE(item->iconName(), KMimeType::iconNameForUrl(item->url()));
+    QVERIFY(item->iconName() != txtIcon);
+}
+
 QTEST_KDEMAIN( ProjectModelTest, GUI)
 #include "projectmodeltest.moc"
 #include "moc_projectmodeltest.cpp"
