@@ -44,6 +44,11 @@ Boston, MA 02110-1301, USA.
 #include <sublime/view.h>
 #include <interfaces/iplugincontroller.h>
 #include <interfaces/iprojectcontroller.h>
+#include <interfaces/ibuddydocumentfinder.h>
+#include <interfaces/iproject.h>
+#include <interfaces/iselectioncontroller.h>
+#include <interfaces/context.h>
+#include <project/projectmodel.h>
 
 #include "core.h"
 #include "mainwindow.h"
@@ -53,9 +58,11 @@ Boston, MA 02110-1301, USA.
 #include "savedialog.h"
 #include <kmessagebox.h>
 #include <KIO/Job>
+#include "workingsetcontroller.h"
 
 #include <config-kdevplatform.h>
-#if HAVE_KOMPARE
+
+#ifdef HAVE_KOMPARE
     #include "patchdocument.h"
 #endif
 
@@ -154,14 +161,33 @@ struct DocumentControllerPrivate {
                 }
 
                 documents.insert(document->url(), document);
+
+                if (!controller->isEmptyDocumentUrl(document->url()))
+                {
+                    fileOpenRecent->addUrl(document->url());
+                }
+
                 break;
             }
         }
     }
 
+    KDevelop::IDocument* findBuddyDocument(const KUrl &url, IBuddyDocumentFinder* finder)
+    {
+        QList<KDevelop::IDocument*> allDocs = controller->openDocuments();
+        foreach( KDevelop::IDocument* doc, allDocs ) {
+            if(finder->areBuddies(url, doc->url())) {
+                return doc;
+            }
+        }
+        return 0;
+    }
+
+
     IDocument* openDocumentInternal( const KUrl & inputUrl, const QString& prefName = QString(),
         const KTextEditor::Range& range = KTextEditor::Range::invalid(), const QString& encoding = "",
-        DocumentController::DocumentActivationParams activationParams = 0 )
+        DocumentController::DocumentActivationParams activationParams = 0,
+        IDocument* buddy = 0)
     {
         IDocument* previousActiveDocument = controller->activeDocument();
         KTextEditor::Cursor previousActivePosition;
@@ -221,7 +247,7 @@ struct DocumentControllerPrivate {
             else
             {
                 //make sure the URL exists
-                if ( !url.isValid() || !KIO::NetAccess::exists( url, KIO::NetAccess::DestinationSide, 0 ) )
+                if ( !url.isValid() || !KIO::NetAccess::exists( url, KIO::NetAccess::SourceSide, ICore::self()->uiController()->activeMainWindow() ) )
                 {
                     kDebug() << "cannot find URL:" << url.url();
                     return 0;
@@ -277,7 +303,8 @@ struct DocumentControllerPrivate {
                     doc = new PartDocument(url, Core::self());
                 } else
                 {
-                    int openAsText = KMessageBox::questionYesNo(0, i18n("KDevelop could not find the editor for file '%1' of type %2.\nDo you want to open it as plain text?", url.fileName(), mimeType->name()), i18n("Could Not Find Editor"));
+                    int openAsText = KMessageBox::questionYesNo(0, i18n("KDevelop could not find the editor for file '%1' of type %2.\nDo you want to open it as plain text?", url.fileName(), mimeType->name()), i18nc("@title:window", "Could Not Find Editor"),
+                                                                KStandardGuiItem::yes(), KStandardGuiItem::no(), "AskOpenWithTextEditor");
                     if (openAsText == KMessageBox::Yes)
                         doc = new TextDocument(url, Core::self(), _encoding);
                     else
@@ -289,7 +316,7 @@ struct DocumentControllerPrivate {
         // The url in the document must equal the current url, else the housekeeping will get broken
         Q_ASSERT(!doc || doc->url() == url);
         
-        if(doc && openDocumentInternal(doc, range, activationParams))
+        if(doc && openDocumentInternal(doc, range, activationParams, buddy))
             return doc;
         else
             return 0;
@@ -298,7 +325,8 @@ struct DocumentControllerPrivate {
     
     bool openDocumentInternal(IDocument* doc,
                                 const KTextEditor::Range& range,
-                                DocumentController::DocumentActivationParams activationParams)
+                                DocumentController::DocumentActivationParams activationParams,
+                                IDocument* buddy = 0)
     {
         IDocument* previousActiveDocument = controller->activeDocument();
         KTextEditor::Cursor previousActivePosition;
@@ -322,7 +350,8 @@ struct DocumentControllerPrivate {
             return false;
         }
         //react on document deletion - we need to cleanup controller structures
-        QObject::connect(sdoc, SIGNAL(aboutToDelete(Sublime::Document*)), controller, SLOT(removeDocument(Sublime::Document*)));
+        
+        QObject::connect(sdoc, SIGNAL(aboutToDelete(Sublime::Document*)), controller, SLOT(notifyDocumentClosed(Sublime::Document*)));
         //We check if it was already opened before
         bool emitOpened = !documents.contains(url);
         if(emitOpened)
@@ -332,9 +361,11 @@ struct DocumentControllerPrivate {
         {
             //find a view if there's one already opened in this area
             Sublime::View *partView = 0;
+            Sublime::AreaIndex* activeViewIdx = area->indexOf(uiController->activeSublimeWindow()->activeView());
             foreach (Sublime::View *view, sdoc->views())
             {
-                if (area->views().contains(view) && (activationParams.testFlag(DocumentController::DoNotForceCurrentView) || area->indexOf(view) == area->indexOf(uiController->activeSublimeWindow()->activeView())))
+                Sublime::AreaIndex* areaIdx = area->indexOf(view);
+                if (areaIdx && areaIdx == activeViewIdx)
                 {
                     partView = view;
                     break;
@@ -349,26 +380,122 @@ struct DocumentControllerPrivate {
             }
             
             KDevelop::TextView* textView = dynamic_cast<KDevelop::TextView*>(partView);
-            if(textView && textView->textView()) {
+            if(textView) {
                 applyRange = false;
-                if (range.isEmpty())
-                    doc->setCursorPosition( range.start() );
-                else
-                    doc->setTextSelection( range );
-            }else if(textView) {
                 textView->setInitialRange(range);
             }
             
             if(addView) {
-                //add view to the area
-                area->addView(partView, uiController->activeSublimeWindow()->activeView());
+                // This code is never executed when restoring session on startup,
+                // only when opening a file manually
+
+                Sublime::View* buddyView = 0;
+                bool placeAfterBuddy = true;
+                if(Core::self()->uiControllerInternal()->arrangeBuddies()) {
+                    // If buddy is not set, look for a (usually) plugin which handles this URL's mimetype
+                    // and use its IBuddyDocumentFinder, if available, to find a buddy document
+                    if(!buddy && doc->mimeType()) {
+                        QString mime = doc->mimeType()->name();
+                        IBuddyDocumentFinder* buddyFinder = IBuddyDocumentFinder::finderForMimeType(mime);
+                        if(buddyFinder) {
+                            buddy = findBuddyDocument(url, buddyFinder);
+                            if(buddy) {
+                                placeAfterBuddy = buddyFinder->buddyOrder(buddy->url(), doc->url());
+                            }
+                        }
+                    }
+
+                    if(buddy) {
+                        Sublime::Document* sublimeDocBuddy = dynamic_cast<Sublime::Document*>(buddy);
+
+                        if(sublimeDocBuddy) {
+                            Sublime::AreaIndex *pActiveViewIndex = area->indexOf(uiController->activeSublimeWindow()->activeView());
+                            if(pActiveViewIndex) {
+                                // try to find existing View of buddy document in current active view's tab
+                                foreach (Sublime::View *pView, pActiveViewIndex->views()) {
+                                    if(sublimeDocBuddy->views().contains(pView)) {
+                                        buddyView = pView;
+                                        break;
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+
+                // add view to the area
+                if(buddyView && area->indexOf(buddyView)) {
+                    if(placeAfterBuddy) {
+                        // Adding new view after buddy view, simple case
+                        area->addView(partView, area->indexOf(buddyView), buddyView);
+                    }
+                    else {
+                        // First new view, then buddy view
+                        area->addView(partView, area->indexOf(buddyView), buddyView);
+                        // move buddyView tab after the new document
+                        area->removeView(buddyView);
+                        area->addView(buddyView, area->indexOf(partView), partView);
+                    }
+                }
+                else {
+                    // no buddy found for new document / plugin does not support buddies / buddy feature disabled
+                    Sublime::View *activeView = uiController->activeSublimeWindow()->activeView();
+                    Sublime::UrlDocument *activeDoc = 0;
+                    IBuddyDocumentFinder *buddyFinder = 0;
+                    if(activeView)
+                        activeDoc = dynamic_cast<Sublime::UrlDocument *>(activeView->document());
+                    if(activeDoc && Core::self()->uiControllerInternal()->arrangeBuddies()) {
+                        QString mime = KMimeType::findByUrl(activeDoc->url())->name();
+                        buddyFinder = IBuddyDocumentFinder::finderForMimeType(mime);
+                    }
+
+                    if(Core::self()->uiControllerInternal()->openAfterCurrent() &&
+                       Core::self()->uiControllerInternal()->arrangeBuddies() &&
+                       buddyFinder)
+                    {
+                        // Check if active document's buddy is directly next to it.
+                        // For example, we have the already-open tabs | *foo.h* | foo.cpp | , foo.h is active.
+                        // When we open a new document here (and the buddy feature is enabled),
+                        // we do not want to separate foo.h and foo.cpp, so we take care and avoid this.
+                        Sublime::AreaIndex *activeAreaIndex = area->indexOf(activeView);
+                        int pos = activeAreaIndex->views().indexOf(activeView);
+                        Sublime::View *afterActiveView = activeAreaIndex->views().value(pos+1, 0);
+
+                        Sublime::UrlDocument *activeDoc = 0, *afterActiveDoc = 0;
+                        if(activeView && afterActiveView) {
+                            activeDoc = dynamic_cast<Sublime::UrlDocument *>(activeView->document());
+                            afterActiveDoc = dynamic_cast<Sublime::UrlDocument *>(afterActiveView->document());
+                        }
+                        if(activeDoc && afterActiveDoc &&
+                           buddyFinder->areBuddies(activeDoc->url(), afterActiveDoc->url()))
+                        {
+                            // don't insert in between of two buddies, but after them
+                            area->addView(partView, activeAreaIndex, afterActiveView);
+                        }
+                        else {
+                            // The active document's buddy is not directly after it
+                            // => no ploblem, insert after active document
+                            area->addView(partView, activeView);
+                        }
+                    }
+                    else {
+                        // Opening as last tab won't disturb our buddies
+                        // Same, if buddies are disabled, we needn't care about them.
+
+                        // this method places the tab according to openAfterCurrent()
+                        area->addView(partView, activeView);
+                    }
+                }
             }
             
             if (!activationParams.testFlag(IDocumentController::DoNotActivate))
             {
                 uiController->activeSublimeWindow()->activateView(partView);
             }
-            fileOpenRecent->addUrl( url );
+            if (!controller->isEmptyDocumentUrl(url))
+            {
+                fileOpenRecent->addUrl( url );
+            }
 
             if( applyRange && range.isValid() )
             {
@@ -426,7 +553,6 @@ struct DocumentControllerPrivate {
     void jumpTo( const HistoryEntry & );*/
 };
 
-
 DocumentController::DocumentController( QObject *parent )
         : IDocumentController( parent )
 {
@@ -439,12 +565,12 @@ DocumentController::DocumentController( QObject *parent )
 
     if(!(Core::self()->setupFlags() & Core::NoUi)) setupActions();
 
-#if HAVE_KOMPARE    
+#ifdef HAVE_KOMPARE
     registerDocumentForMimetype("text/x-patch", new PatchDocumentFactory);
 #endif
 }
 
-void KDevelop::DocumentController::initialize()
+void DocumentController::initialize()
 {
 }
 
@@ -455,9 +581,8 @@ void DocumentController::cleanup()
 
     // Close all documents without checking if they should be saved.
     // This is because the user gets a chance to save them during MainWindow::queryClose.
-    foreach (Sublime::MainWindow* mw, Core::self()->uiControllerInternal()->mainWindows())
-        foreach (IDocument* doc, documentsInWindow(dynamic_cast<KDevelop::MainWindow*>(mw)))
-            doc->close(IDocument::Discard);
+    foreach (IDocument* doc, openDocuments())
+        doc->close(IDocument::Discard);
 }
 
 DocumentController::~DocumentController()
@@ -476,27 +601,28 @@ void DocumentController::setupActions()
     action->setIcon(KIcon("document-open"));
     action->setShortcut( Qt::CTRL + Qt::Key_O );
     action->setText(i18n( "&Open..." ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( chooseDocument() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(chooseDocument()) );
     action->setToolTip( i18n( "Open file" ) );
     action->setWhatsThis( i18n( "<b>Open file</b><p>Opens a file for editing.</p>" ) );
 
     d->fileOpenRecent = KStandardAction::openRecent(this,
-                    SLOT(slotOpenDocument(const KUrl&)), ac);
+                    SLOT(slotOpenDocument(KUrl)), ac);
     d->fileOpenRecent->setWhatsThis(i18n("This lists files which you have opened recently, and allows you to easily open them again."));
     d->fileOpenRecent->loadEntries( KConfigGroup(KGlobal::config(), "Recent Files" ) );
 
     action = d->saveAll = ac->addAction( "file_save_all" );
     action->setIcon(KIcon("document-save"));
     action->setText(i18n( "Save Al&l" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( slotSaveAllDocuments() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(slotSaveAllDocuments()) );
     action->setToolTip( i18n( "Save all open documents" ) );
     action->setWhatsThis( i18n( "<b>Save all documents</b><p>Save all open documents, prompting for additional information when necessary.</p>" ) );
+    action->setShortcut( QKeySequence(Qt::CTRL + Qt::Key_L) );
     action->setEnabled(false);
 
     action = d->revertAll = ac->addAction( "file_revert_all" );
     action->setIcon(KIcon("document-revert"));
-    action->setText(i18n( "Rever&t All" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( reloadAllDocuments() ) );
+    action->setText(i18n( "Reload All" ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(reloadAllDocuments()) );
     action->setToolTip( i18n( "Revert all open documents" ) );
     action->setWhatsThis( i18n( "<b>Revert all documents</b><p>Revert all open documents, returning to the previously saved state.</p>" ) );
     action->setEnabled(false);
@@ -505,7 +631,7 @@ void DocumentController::setupActions()
     action->setIcon(KIcon("window-close"));
     action->setShortcut( Qt::CTRL + Qt::Key_W );
     action->setText( i18n( "&Close" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( fileClose() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(fileClose()) );
     action->setToolTip( i18n( "Close File" ) );
     action->setWhatsThis( i18n( "<b>Close File</b><p>Closes current file.</p>" ) );
     action->setEnabled(false);
@@ -513,7 +639,7 @@ void DocumentController::setupActions()
     action = d->closeAll = ac->addAction( "file_close_all" );
     action->setIcon(KIcon("window-close"));
     action->setText(i18n( "Clos&e All" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( closeAllDocuments() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(closeAllDocuments()) );
     action->setToolTip( i18n( "Close all open documents" ) );
     action->setWhatsThis( i18n( "<b>Close all documents</b><p>Close all open documents, prompting for additional information when necessary.</p>" ) );
     action->setEnabled(false);
@@ -522,7 +648,7 @@ void DocumentController::setupActions()
     action->setIcon(KIcon("window-close"));
     action->setShortcut( Qt::CTRL + Qt::SHIFT + Qt::Key_W );
     action->setText(i18n( "Close All Ot&hers" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( closeAllOtherDocuments() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(closeAllOtherDocuments()) );
     action->setToolTip( i18n( "Close all other documents" ) );
     action->setWhatsThis( i18n( "<b>Close all other documents</b><p>Close all open documents, with the exception of the currently active document.</p>" ) );
     action->setEnabled(false);
@@ -546,10 +672,21 @@ void DocumentController::slotOpenDocument(const KUrl &url)
 IDocument* DocumentController::openDocumentFromText( const QString& data )
 {
     IDocument* d = openDocument(nextEmptyDocumentUrl());
+    Q_ASSERT(d->textDocument());
     d->textDocument()->setText( data );
     return d;
 }
 
+bool DocumentController::openDocumentFromTextSimple( QString text )
+{
+    return (bool)openDocumentFromText( text );
+}
+
+bool DocumentController::openDocumentSimple( QString url )
+{
+    return (bool)openDocument( KUrl(url) );
+}
+
 IDocument* DocumentController::openDocument( const KUrl& inputUrl, const QString& prefName )
 {
     return d->openDocumentInternal( inputUrl, prefName );
@@ -558,17 +695,18 @@ IDocument* DocumentController::openDocument( const KUrl& inputUrl, const QString
 IDocument* DocumentController::openDocument( const KUrl & inputUrl,
         const KTextEditor::Range& range,
         DocumentActivationParams activationParams,
-        const QString& encoding)
+        const QString& encoding, IDocument* buddy)
 {
-    return d->openDocumentInternal( inputUrl, "", range, encoding, activationParams );
+    return d->openDocumentInternal( inputUrl, "", range, encoding, activationParams, buddy);
 }
 
 
 bool DocumentController::openDocument(IDocument* doc,
                                       const KTextEditor::Range& range,
-                                      DocumentActivationParams activationParams)
+                                      DocumentActivationParams activationParams,
+                                      IDocument* buddy)
 {
-    return d->openDocumentInternal( doc, range, activationParams );
+    return d->openDocumentInternal( doc, range, activationParams, buddy);
 }
 
 
@@ -595,10 +733,13 @@ void DocumentController::closeDocument( const KUrl &url )
     d->documents[url]->close();
 }
 
-void DocumentController::notifyDocumentClosed(IDocument* doc)
+void DocumentController::notifyDocumentClosed(Sublime::Document* doc_)
 {
-    d->documents.remove(doc->url());
-
+    IDocument* doc = dynamic_cast<IDocument*>(doc_);
+    Q_ASSERT(doc);
+    
+    d->removeDocument(doc_);
+    
     if (d->documents.isEmpty()) {
         if (d->saveAll)
             d->saveAll->setEnabled(false);
@@ -620,10 +761,7 @@ IDocument * DocumentController::documentForUrl( const KUrl & dirtyUrl ) const
     //Fix urls that might not be absolute
     KUrl url(dirtyUrl);
     url.cleanPath();
-    if ( d->documents.contains( url ) )
-        return d->documents.value( url );
-
-    return 0;
+    return d->documents.value( url, 0 );
 }
 
 QList<IDocument*> DocumentController::openDocuments() const
@@ -663,7 +801,7 @@ bool KDevelop::DocumentController::saveSomeDocuments(const QList< IDocument * >
 {
     if (mode & IDocument::Silent) {
         foreach (IDocument* doc, modifiedDocuments(list)) {
-            if( !doc->save(mode) )
+            if( !DocumentController::isEmptyDocumentUrl(doc->url()) && !doc->save(mode) )
             {
                 if( doc )
                     qWarning() << "!! Could not save document:" << doc->url();
@@ -687,9 +825,10 @@ bool KDevelop::DocumentController::saveSomeDocuments(const QList< IDocument * >
     return true;
 }
 
-QList< IDocument * > KDevelop::DocumentController::documentsInWindow(MainWindow * mw) const
+QList< IDocument * > KDevelop::DocumentController::visibleDocumentsInWindow(MainWindow * mw) const
 {
     // Gather a list of all documents which do have a view in the given main window
+    // Does not find documents which are open in inactive areas
     QList<IDocument*> list;
     foreach (IDocument* doc, openDocuments()) {
         if (Sublime::Document* sdoc = dynamic_cast<Sublime::Document*>(doc)) {
@@ -735,9 +874,9 @@ QList< IDocument * > KDevelop::DocumentController::modifiedDocuments(const QList
     return ret;
 }
 
-bool DocumentController::saveAllDocumentsForWindow(KDevelop::MainWindow* mw, KDevelop::IDocument::DocumentSaveMode mode, bool currentAreaOnly)
+bool DocumentController::saveAllDocumentsForWindow(KParts::MainWindow* mw, KDevelop::IDocument::DocumentSaveMode mode, bool currentAreaOnly)
 {
-    QList<IDocument*> checkSave = documentsExclusivelyInWindow(mw, currentAreaOnly);
+    QList<IDocument*> checkSave = documentsExclusivelyInWindow(dynamic_cast<KDevelop::MainWindow*>(mw), currentAreaOnly);
 
     return saveSomeDocuments(checkSave, mode);
 }
@@ -745,21 +884,22 @@ bool DocumentController::saveAllDocumentsForWindow(KDevelop::MainWindow* mw, KDe
 void DocumentController::reloadAllDocuments()
 {
     if (Sublime::MainWindow* mw = Core::self()->uiControllerInternal()->activeSublimeWindow()) {
-        QList<IDocument*> views = documentsInWindow(dynamic_cast<KDevelop::MainWindow*>(mw));
+        QList<IDocument*> views = visibleDocumentsInWindow(dynamic_cast<KDevelop::MainWindow*>(mw));
 
         if (!saveSomeDocuments(views, IDocument::Default))
             // User cancelled or other error
             return;
 
         foreach (IDocument* doc, views)
-            doc->reload();
+            if(!isEmptyDocumentUrl(doc->url()))
+                doc->reload();
     }
 }
 
 void DocumentController::closeAllDocuments()
 {
     if (Sublime::MainWindow* mw = Core::self()->uiControllerInternal()->activeSublimeWindow()) {
-        QList<IDocument*> views = documentsInWindow(dynamic_cast<KDevelop::MainWindow*>(mw));
+        QList<IDocument*> views = visibleDocumentsInWindow(dynamic_cast<KDevelop::MainWindow*>(mw));
 
         if (!saveSomeDocuments(views, IDocument::Default))
             // User cancelled or other error
@@ -803,6 +943,43 @@ IDocument* DocumentController::activeDocument() const
     return dynamic_cast<IDocument*>(uiController->activeSublimeWindow()->activeView()->document());
 }
 
+QString DocumentController::activeDocumentPath( QString target ) const
+{
+    if(target.size()) {
+        foreach(IProject* project, Core::self()->projectController()->projects()) {
+            if(project->name().startsWith(target, Qt::CaseInsensitive)) {
+                return project->folder().pathOrUrl() + "/.";
+            }
+        }
+    }
+    IDocument* doc = activeDocument();
+    if(!doc || target == "[selection]")
+    {
+        Context* selection = ICore::self()->selectionController()->currentSelection();
+        if(selection && selection->type() == Context::ProjectItemContext && static_cast<ProjectItemContext*>(selection)->items().size())
+        {
+            QString ret = static_cast<ProjectItemContext*>(selection)->items()[0]->url().pathOrUrl();
+            if(static_cast<ProjectItemContext*>(selection)->items()[0]->folder())
+                ret += "/.";
+            return  ret;
+        }
+        return QString();
+    }
+    return doc->url().pathOrUrl();
+}
+
+QStringList DocumentController::activeDocumentPaths() const
+{
+    UiController *uiController = Core::self()->uiControllerInternal();
+    if( !uiController->activeSublimeWindow() ) return QStringList();
+    
+    QSet<QString> documents;
+    foreach(Sublime::View* view, uiController->activeSublimeWindow()->area()->views())
+        documents.insert(KUrl(view->document()->documentSpecifier()).pathOrUrl());
+    
+    return documents.toList();
+}
+
 void DocumentController::registerDocumentForMimetype( const QString& mimetype,
                                         KDevelop::IDocumentFactory* factory )
 {
@@ -856,6 +1033,160 @@ KTextEditor::Document* DocumentController::globalTextEditorInstance()
     return d->globalTextEditorInstance;
 }
 
+bool DocumentController::openDocumentsSimple( QStringList urls )
+{
+    Sublime::Area* area = Core::self()->uiControllerInternal()->activeArea();
+    Sublime::AreaIndex* areaIndex = area->rootIndex();
+
+    QList<Sublime::View*> topViews = static_cast<Sublime::MainWindow*>(Core::self()->uiControllerInternal()->activeMainWindow())->getTopViews();
+    
+    if(Sublime::View* activeView = Core::self()->uiControllerInternal()->activeSublimeWindow()->activeView())
+        areaIndex = area->indexOf(activeView);
+
+    kDebug() << "opening " << urls << " to area " << area << " index " << areaIndex << " with children " << areaIndex->first() << " " << areaIndex->second();
+    
+    bool isFirstView = true;
+    
+    bool ret = openDocumentsWithSplitSeparators( areaIndex, urls, isFirstView );
+    
+    kDebug() << "area arch. after opening: " << areaIndex->print();
+    
+    // Required because sublime sometimes doesn't update correctly when the area-index contents has been changed
+    // (especially when views have been moved to other indices, through unsplit, split, etc.)
+    static_cast<Sublime::MainWindow*>(Core::self()->uiControllerInternal()->activeMainWindow())->reconstructViews(topViews);
+    
+    return ret;
+}
+
+bool DocumentController::openDocumentsWithSplitSeparators( Sublime::AreaIndex* index, QStringList urlsWithSeparators, bool& isFirstView )
+{
+    kDebug() << "opening " << urlsWithSeparators << " index " << index << " with children " << index->first() << " " << index->second() << " view-count " << index->viewCount();
+    if(urlsWithSeparators.isEmpty())
+        return true;
+    
+    Sublime::Area* area = Core::self()->uiControllerInternal()->activeArea();
+    
+    QList<int> topLevelSeparators; // Indices of the top-level separators (with groups skipped)
+    QStringList separators = QStringList() << "/" << "-";
+    QList<QStringList> groups;
+    
+    bool ret = true;
+    
+    {
+        int parenDepth = 0;
+        int groupStart = 0;
+        for(int pos = 0; pos < urlsWithSeparators.size(); ++pos)
+        {
+            QString item = urlsWithSeparators[pos];
+            if(separators.contains(item))
+            {
+                if(parenDepth == 0)
+                    topLevelSeparators << pos;
+            }else if(item == "[")
+            {
+                if(parenDepth == 0)
+                    groupStart = pos+1;
+                ++parenDepth;
+            }
+            else if(item == "]")
+            {
+                if(parenDepth > 0)
+                {
+                    --parenDepth;
+                    
+                    if(parenDepth == 0)
+                        groups << urlsWithSeparators.mid(groupStart, pos-groupStart);
+                }
+                else{
+                    kDebug() << "syntax error in " << urlsWithSeparators << ": parens do not match";
+                    ret = false;
+                }
+            }else if(parenDepth == 0)
+            {
+                groups << (QStringList() << item);
+            }
+        }
+    }
+    
+    if(topLevelSeparators.isEmpty())
+    {
+        if(urlsWithSeparators.size() > 1)
+        {
+            foreach(QStringList group, groups)
+                ret &= openDocumentsWithSplitSeparators( index, group, isFirstView );
+        }else{
+            while(index->isSplit())
+                index = index->first();
+            // Simply open the document into the area index
+            IDocument* doc = Core::self()->documentControllerInternal()->openDocument(KUrl(urlsWithSeparators.front()),
+                        KTextEditor::Cursor::invalid(), IDocumentController::DoNotActivate | IDocumentController::DoNotCreateView);
+            Sublime::Document *sublimeDoc = dynamic_cast<Sublime::Document*>(doc);
+            if (sublimeDoc) {
+                Sublime::View* view = sublimeDoc->createView();
+                area->addView(view, index);
+                if(isFirstView)
+                {
+                    static_cast<Sublime::MainWindow*>(Core::self()->uiControllerInternal()->activeMainWindow())->activateView(view);
+                    isFirstView = false;
+                }
+            }else{
+                ret = false;
+            }
+        }
+        return ret;
+    }
+    
+    // Pick a separator in the middle
+    
+    int pickSeparator = topLevelSeparators[topLevelSeparators.size()/2];
+    
+    bool activeViewToSecondChild = false;
+    if(pickSeparator == urlsWithSeparators.size()-1)
+    {
+        // There is no right child group, so the right side should be filled with the currently active views
+        activeViewToSecondChild = true;
+    }else{
+        QStringList separatorsAndParens = separators;
+        separatorsAndParens << "[" << "]";
+        // Check if the second child-set contains an unterminated separator, which means that the active views should end up there
+        for(int pos = pickSeparator+1; pos < urlsWithSeparators.size(); ++pos)
+            if( separators.contains(urlsWithSeparators[pos]) && (pos == urlsWithSeparators.size()-1 ||
+                separatorsAndParens.contains(urlsWithSeparators[pos-1]) ||
+                separatorsAndParens.contains(urlsWithSeparators[pos-1])) )
+                    activeViewToSecondChild = true;
+    }
+    
+    Qt::Orientation orientation = urlsWithSeparators[pickSeparator] == "/" ? Qt::Horizontal : Qt::Vertical;
+    
+    if(!index->isSplit())
+    {
+        kDebug() << "splitting " << index << "orientation" << orientation << "to second" << activeViewToSecondChild;
+        index->split(orientation, activeViewToSecondChild);
+    }else{
+        index->setOrientation(orientation);
+        kDebug() << "WARNING: Area is already split (shouldn't be)" << urlsWithSeparators;
+    }
+    
+    openDocumentsWithSplitSeparators( index->first(), urlsWithSeparators.mid(0, pickSeparator) , isFirstView );
+    if(pickSeparator != urlsWithSeparators.size() - 1)
+        openDocumentsWithSplitSeparators( index->second(), urlsWithSeparators.mid(pickSeparator+1, urlsWithSeparators.size() - (pickSeparator+1) ), isFirstView );
+    
+    // Clean up the child-indices, because document-loading may fail
+    
+    if(!index->first()->viewCount() && !index->first()->isSplit())
+    {
+        kDebug() << "unsplitting first";
+        index->unsplit(index->first());
+    }
+    else if(!index->second()->viewCount() && !index->second()->isSplit())
+    {
+        kDebug() << "unsplitting second";
+        index->unsplit(index->second());
+    }
+
+    return ret;
+}
+
 
 }
 
