@@ -113,10 +113,10 @@ QString ProjectProgress::statusName() const
 
 void ProjectProgress::setBuzzy()
 {
-    kDebug() << "showing busy prorgess" << statusName();
+    kDebug() << "showing busy progress" << statusName();
     // show an indeterminate progressbar
     emit showProgress(this, 0,0,0);
-    emit showMessage(this, i18n("Loading %1", projectName));
+    emit showMessage(this, i18nc("%1: Project name", "Loading %1", projectName));
 }
 
 
@@ -153,14 +153,21 @@ public:
     bool scheduleReload;
     ProjectProgress* progress;
 
-    void reloadDone()
+    void reloadDone(KJob* job)
     {
         progress->setDone();
         loading = false;
-        Core::self()->projectController()->projectModel()->appendRow(topItem);
-        if (scheduleReload) {
-            scheduleReload = false;
-            project->reloadModel();
+
+        ProjectController* projCtrl = Core::self()->projectControllerInternal();
+        if (job->errorText().isEmpty() && !Core::self()->shuttingDown()) {
+            projCtrl->projectModel()->appendRow(topItem);
+
+            if (scheduleReload) {
+                scheduleReload = false;
+                project->reloadModel();
+            }
+        } else {
+            projCtrl->abortOpeningProject(project);
         }
     }
 
@@ -487,8 +494,10 @@ void Project::reloadModel()
     d->loading = true;
     d->fileSet.clear();
 
+    // delete topItem and remove it from model
     ProjectModel* model = Core::self()->projectController()->projectModel();
     model->removeRow( d->topItem->row() );
+    d->topItem = 0;
 
     IProjectFileManager* iface = d->manager->extension<IProjectFileManager>();
     if (!d->importTopItem(iface))
@@ -500,7 +509,7 @@ void Project::reloadModel()
 
     d->progress->setBuzzy();
     KJob* importJob = iface->createImportJob(d->topItem );
-    connect(importJob, SIGNAL(finished(KJob*)), SLOT(reloadDone()));
+    connect(importJob, SIGNAL(finished(KJob*)), SLOT(reloadDone(KJob*)));
     Core::self()->runController()->registerJob( importJob );
 }
 
@@ -537,6 +546,12 @@ bool Project::open( const KUrl& projectFileUrl_ )
 
 void Project::close()
 {
+    Q_ASSERT(d->topItem);
+    if (d->topItem->row() == -1) {
+        kWarning() << "Something went wrong. ProjectFolderItem detached. Project closed during reload?";
+        return;
+    }
+
     Core::self()->projectController()->projectModel()->removeRow( d->topItem->row() );
 
     if( !KIO::NetAccess::upload( d->developerTempFile, d->developerFileUrl,
@@ -657,12 +672,23 @@ IPlugin* Project::versionControlPlugin() const
 
 void Project::addToFileSet( const IndexedString& file )
 {
+    if (d->fileSet.contains(file)) {
+        return;
+    }
+
     d->fileSet.insert( file );
+    emit fileAddedToSet( this, file );
 }
 
 void Project::removeFromFileSet( const IndexedString& file )
 {
-    d->fileSet.remove( file );
+    QSet<IndexedString>::iterator it = d->fileSet.find(file);
+    if (it == d->fileSet.end()) {
+        return;
+    }
+
+    d->fileSet.erase( it );
+    emit fileRemovedFromSet( this, file );
 }
 
 QSet<IndexedString> Project::fileSet() const
