@@ -280,9 +280,14 @@ void SourceFormatterSettings::enableStyleButtons()
                      && styleList->currentItem()->data( STYLE_ROLE ).toString().startsWith( userStylePrefix );
 
     QString languageName = cbLanguages->currentText();
-    LanguageSettings& l = languages[languageName];
-    ISourceFormatter* fmt = l.selectedFormatter->formatter;
-    bool hasEditWidget = ( fmt && fmt->editStyleWidget( l.mimetypes.first() ) );
+    QMap< QString, LanguageSettings >::const_iterator it = languages.constFind(languageName);
+    bool hasEditWidget = false;
+    if (it != languages.constEnd()) {
+        const LanguageSettings& l = it.value();
+        Q_ASSERT(l.selectedFormatter);
+        ISourceFormatter* fmt = l.selectedFormatter->formatter;
+        hasEditWidget = ( fmt && fmt->editStyleWidget( l.mimetypes.first() ) );
+    }
     btnDelStyle->setEnabled( userEntry );
     btnEditStyle->setEnabled( userEntry && hasEditWidget );
     btnNewStyle->setEnabled( cbFormatters->currentIndex() >= 0 && hasEditWidget );
@@ -376,8 +381,8 @@ void SourceFormatterSettings::deleteStyle()
     }
     if (!otherLanguageNames.empty() &&
         KMessageBox::warningContinueCancel(this,
-        i18n("The style %1 is also used for the following languages:\n%2.\nAre you sure you want to delete it?")
-        .arg(styleIter.value()->caption()).arg(otherLanguageNames.join("\n")), i18n("Style being deleted")) != KMessageBox::Continue) {
+        i18n("The style %1 is also used for the following languages:\n%2.\nAre you sure you want to delete it?",
+        styleIter.value()->caption(), otherLanguageNames.join("\n")), i18n("Style being deleted")) != KMessageBox::Continue) {
         return;
     }
     styleList->takeItem( styleList->currentRow() );
