@@ -67,6 +67,19 @@ namespace KDevelop {
 
 const int MAX_DOC_SETTINGS = 20;
 
+// This sets cursor position and selection on the view to the given
+// range. Selection is set only for non-empty ranges
+// Factored into a function since its needed in 3 places already
+static void selectAndReveal( KTextEditor::View* view, const KTextEditor::Range& range ) {
+    Q_ASSERT(view);
+    if (range.isValid()) {
+        view->setCursorPosition(range.start());
+        if (!range.isEmpty()) {
+            view->setSelection(range);
+        }
+    }
+}
+
 struct TextDocumentPrivate {
     TextDocumentPrivate(TextDocument *textDocument)
         : encoding(""), m_textDocument(textDocument)
@@ -292,6 +305,7 @@ public:
     KStatusBar *statusBar;
     QLabel* statusLabel;
     QString status;
+    const TextView* textView;
 
 };
 
@@ -346,6 +360,8 @@ QWidget *TextDocument::createViewWidget(QWidget *parent)
         if (!url().isEmpty() && !DocumentController::isEmptyDocumentUrl(url()))
             d->document->openUrl( url() );
 
+        d->setStatus(d->document, false);
+
         /* It appears, that by default a part will be deleted the the
            first view containing it is deleted.  Since we do want
            to have several views, disable that behaviour.  */
@@ -425,7 +441,14 @@ void TextDocument::reload()
     if (!d->document)
         return;
 
+    KTextEditor::ModificationInterface* modif=0;
+    if(d->state==Dirty) {
+        modif = qobject_cast<KTextEditor::ModificationInterface*>(d->document);
+        modif->setModifiedOnDiskWarning(false);
+    }
     d->document->documentReload();
+    if(modif)
+        modif->setModifiedOnDiskWarning(true);
 }
 
 bool TextDocument::save(DocumentSaveMode mode)
@@ -447,7 +470,7 @@ bool TextDocument::save(DocumentSaveMode mode)
                 int code = KMessageBox::warningYesNoCancel(
                     Core::self()->uiController()->activeMainWindow(),
                     i18n("The document \"%1\" has unsaved changes. Would you like to save them?", d->document->url().toLocalFile()),
-                    i18n("Close Document"));
+                    i18nc("@title:window", "Close Document"));
                 if (code != KMessageBox::Yes)
                     return false;
             }
@@ -462,7 +485,7 @@ bool TextDocument::save(DocumentSaveMode mode)
                     i18n("The file \"%1\" is modified on disk.\n\nAre "
                         "you sure you want to overwrite it? (External "
                         "changes will be lost.)", d->document->url().toLocalFile()),
-                    i18n("Document Externally Modified"));
+                    i18nc("@title:window", "Document Externally Modified"));
                 if (code != KMessageBox::Yes)
                     return false;
             }
@@ -584,8 +607,7 @@ void TextDocument::setTextSelection(const KTextEditor::Range &range)
     KTextEditor::View *view = d->document->activeView();
 
     if (view) {
-        view->setCursorPosition(range.start());
-        view->setSelection( range );
+        selectAndReveal(view, range);
     }
 }
 
@@ -626,21 +648,9 @@ KDevelop::TextView::~TextView()
 
 QWidget * KDevelop::TextView::createWidget(QWidget * parent)
 {
-    TextDocument* doc = static_cast<TextDocument*>(document());
-    KTextEditor::View* view = static_cast<KTextEditor::View*>(doc->createViewWidget(parent));
-    if(d->initialRange.isValid()) {
-        if(d->initialRange.isEmpty())
-            view->setCursorPosition(d->initialRange.start());
-        else {
-            view->setCursorPosition(d->initialRange.end());
-            view->setSelection(d->initialRange);
-        }
-    }
-    TextEditorWidget* teWidget = new TextEditorWidget(parent);
-    teWidget->setEditorView(view);
-    connect(teWidget, SIGNAL(statusChanged()),
-            this, SLOT(sendStatusChanged()));
-            
+    TextEditorWidget* teWidget = new TextEditorWidget(this, parent);
+    connect(teWidget, SIGNAL(statusChanged()), this, SLOT(sendStatusChanged()));
+
     d->editor = teWidget;
     connect(d->editor, SIGNAL(destroyed(QObject*)), this, SLOT(editorDestroyed(QObject*)));
     
@@ -665,15 +675,27 @@ QString KDevelop::TextView::viewState() const
         } else {
             KTextEditor::Cursor cursor = d->editor->editorView()->cursorPosition();
             return QString("Cursor=%1,%2").arg(cursor.line()).arg(cursor.column());
-        }    }else
+        }
+    }
+    else
     {
         kDebug() << "TextView's internal KTE view disappeared!";
         return QString();
     }
 }
 
-void KDevelop::TextView::setInitialRange(KTextEditor::Range range) {
-    d->initialRange = range;
+void KDevelop::TextView::setInitialRange(const KTextEditor::Range& range)
+{
+    if(d->editor && d->editor->isInitialized()) {
+        selectAndReveal(d->editor->editorView(), range);
+    } else {
+        d->initialRange = range;
+    }
+}
+
+KTextEditor::Range KDevelop::TextView::initialRange() const
+{
+    return d->initialRange;
 }
 
 void KDevelop::TextView::setState(const QString & state)
@@ -681,17 +703,10 @@ void KDevelop::TextView::setState(const QString & state)
     static QRegExp reCursor("Cursor=([\\d]+),([\\d]+)");
     static QRegExp reSelection("Selection=([\\d]+),([\\d]+),([\\d]+),([\\d]+)");
     if (reCursor.exactMatch(state)) {
-        if (d->editor && d->editor->editorView())
-            d->editor->editorView()->setCursorPosition(KTextEditor::Cursor(reCursor.cap(1).toInt(), reCursor.cap(2).toInt()));
-        else
-            setInitialRange(KTextEditor::Range(KTextEditor::Cursor(reCursor.cap(1).toInt(), reCursor.cap(2).toInt()), 0));
+        setInitialRange(KTextEditor::Range(KTextEditor::Cursor(reCursor.cap(1).toInt(), reCursor.cap(2).toInt()), 0));
     } else if (reSelection.exactMatch(state)) {
         KTextEditor::Range range(reSelection.cap(1).toInt(), reSelection.cap(2).toInt(), reSelection.cap(3).toInt(), reSelection.cap(4).toInt());
-        if (d->editor && d->editor->editorView()) {
-            d->editor->editorView()->setCursorPosition(range.end());
-            d->editor->editorView()->setSelection(range);
-        } else
-            setInitialRange(range);
+        setInitialRange(range);
     }
 }
 
@@ -700,6 +715,18 @@ QString KDevelop::TextDocument::documentType() const
     return "Text";
 }
 
+QIcon KDevelop::TextDocument::defaultIcon() const
+{
+    if (d->document) {
+        KMimeType::Ptr mime = KMimeType::mimeType(d->document->mimeType());
+        KIcon icon(mime->iconName());
+        if (!icon.isNull()) {
+            return icon;
+        }
+    }
+    return PartDocument::defaultIcon();
+}
+
 KTextEditor::View *KDevelop::TextView::textView() const
 {  
     if (d->editor)
@@ -721,14 +748,17 @@ void KDevelop::TextView::sendStatusChanged()
     emit statusChanged(this);
 }
 
-KDevelop::TextEditorWidget::TextEditorWidget(QWidget* parent)
+KDevelop::TextEditorWidget::TextEditorWidget(const TextView* view, QWidget* parent)
 : QWidget(parent), KXMLGUIClient(), d(new TextEditorWidgetPrivate)
 {
     d->widgetLayout = new QVBoxLayout(this);
     d->widgetLayout->setMargin(0);
     d->widgetLayout->setSpacing(0);
+    d->textView = view;
+    d->view = 0;
 
     setLayout(d->widgetLayout);
+    initialize();
 }
 
 KDevelop::TextEditorWidget::~TextEditorWidget()
@@ -736,6 +766,17 @@ KDevelop::TextEditorWidget::~TextEditorWidget()
     delete d;
 }
 
+void KDevelop::TextEditorWidget::initialize()
+{
+    if(d->view)
+        return;
+    TextDocument* doc = static_cast<TextDocument*>(d->textView->document());
+    KTextEditor::View* view = qobject_cast<KTextEditor::View*>(doc->createViewWidget(this));
+    KTextEditor::Range ir = d->textView->initialRange();
+    selectAndReveal(view, ir);
+    setEditorView(view);
+}
+
 void KDevelop::TextEditorWidget::viewStatusChanged(KTextEditor::View* view, const KTextEditor::Cursor& )
 {
     // This fetches the virtual cursor, which expands tab characters properly, i.e. instead of col == 1
@@ -769,9 +810,23 @@ QString KDevelop::TextEditorWidget::status() const
     return d->status;
 }
 
-KTextEditor::View* KDevelop::TextEditorWidget::editorView() const
+KTextEditor::View* KDevelop::TextEditorWidget::editorView()
 {
+    if(!d->view)
+        initialize();
     return d->view;
 }
 
+bool KDevelop::TextEditorWidget::isInitialized() const
+{
+    return d->view!=0;
+}
+
+void KDevelop::TextEditorWidget::showEvent(QShowEvent* event)
+{
+    if(!d->view)
+        initialize();
+    QWidget::showEvent(event);
+}
+
 #include "textdocument.moc"
