@@ -28,6 +28,7 @@
 #include <kdebug.h>
 #include <kdialog.h>
 #include <klocale.h>
+#include <kmenubar.h>
 #include <ksettings/dialog.h>
 #include <ksettings/dispatcher.h>
 #include <kcmultidialog.h>
@@ -50,15 +51,17 @@
 #include <kactioncollection.h>
 #include <ktexteditor/view.h>
 #include "workingsetcontroller.h"
+#include "workingsets/workingset.h"
 
 namespace KDevelop {
 
 class UiControllerPrivate {
 public:
     UiControllerPrivate(UiController *controller)
-    : cfgDlg(0), areasRestored(false), m_controller(controller)
+    : areasRestored(false), m_controller(controller)
     {
-        Core::self()->workingSetControllerInternal()->initializeController(m_controller);
+        if (Core::self()->workingSetControllerInternal())
+            Core::self()->workingSetControllerInternal()->initializeController(m_controller);
 
         QMap<QString, Sublime::Position> desired;
 
@@ -71,6 +74,7 @@ public:
         desired["org.kdevelop.ContextBrowser"] = Sublime::Bottom;
         desired["org.kdevelop.KonsoleView"] = Sublime::Bottom;
         desired["org.kdevelop.SnippetView"] = Sublime::Right;
+        desired["org.kdevelop.ExternalScriptView"] = Sublime::Right;
         Sublime::Area* a =
             new Sublime::Area(m_controller, "code", i18n("Code"));
         a->setDesiredToolViews(desired);
@@ -125,10 +129,7 @@ public:
 
     QMap<IToolViewFactory*, Sublime::ToolDocument*> factoryDocuments;
 
-    KSettings::Dialog* cfgDlg;
-
     Sublime::MainWindow* activeSublimeWindow;
-    QList<Sublime::MainWindow*> sublimeWindows;
     bool areasRestored;
 
     //Currently shown assistant popup.
@@ -149,6 +150,10 @@ public:
         return m_factory->create(parent);
     }
 
+    virtual QList< QAction* > contextMenuActions(QWidget* viewWidget) const
+    {
+        return m_factory->contextMenuActions( viewWidget );
+    }
 
     QList<QAction*> toolBarActions( QWidget* viewWidget ) const
     {
@@ -168,6 +173,34 @@ public:
     IToolViewFactory *factory;
 };
 
+
+class NewToolViewListWidget: public QListWidget {
+    Q_OBJECT
+
+public:
+    NewToolViewListWidget(MainWindow *mw, QWidget* parent = 0)
+        :QListWidget(parent), m_mw(mw)
+    {
+        connect(this, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(addNewToolViewByDoubleClick(QModelIndex)));
+    }
+
+Q_SIGNALS:
+    void addNewToolView(MainWindow *mw, QListWidgetItem *item);
+
+private Q_SLOTS:
+    void addNewToolViewByDoubleClick(QModelIndex index)
+    {
+        QListWidgetItem *item = itemFromIndex(index);
+        // Disable item so that the toolview can not be added again.
+        item->setFlags(item->flags() & ~Qt::ItemIsEnabled);
+        emit addNewToolView(m_mw, item);
+    }
+
+private:
+    MainWindow *m_mw;
+};
+
+
 UiController::UiController(Core *core)
     :Sublime::Controller(0), IUiController(), d(new UiControllerPrivate(this))
 {
@@ -178,35 +211,10 @@ UiController::UiController(Core *core)
         return;
 
     connect( QApplication::instance(),
-             SIGNAL( focusChanged( QWidget*, QWidget* ) ),
-            this, SLOT( widgetChanged( QWidget*, QWidget* ) ) );
+             SIGNAL(focusChanged(QWidget*,QWidget*)),
+            this, SLOT(widgetChanged(QWidget*,QWidget*)) );
 
-    KActionCollection * actions = defaultMainWindow()->actionCollection();
-
-    KAction* assistantaction1 = actions->addAction("assistant_action_1");
-    assistantaction1->setText( i18n("&Assistant Action 1") );
-    assistantaction1->setShortcut( Qt::ALT | Qt::Key_1 );
-    connect(assistantaction1, SIGNAL(triggered(bool)), this, SLOT(assistantAction1()));
-
-    KAction* assistantaction2 = actions->addAction("assistant_action_2");
-    assistantaction2->setText( i18n("&Assistant Action 2") );
-    assistantaction2->setShortcut( Qt::ALT | Qt::Key_2 );
-    connect(assistantaction2, SIGNAL(triggered(bool)), this, SLOT(assistantAction2(bool)));
-
-    KAction* assistantaction3 = actions->addAction("assistant_action_3");
-    assistantaction3->setText( i18n("&Assistant Action 3") );
-    assistantaction3->setShortcut( Qt::ALT | Qt::Key_3 );
-    connect(assistantaction3, SIGNAL(triggered(bool)), this, SLOT(assistantAction3(bool)));
-
-    KAction* assistantaction4 = actions->addAction("assistant_action_4");
-    assistantaction4->setText( i18n("&Assistant Action 4") );
-    assistantaction4->setShortcut( Qt::ALT | Qt::Key_4 );
-    connect(assistantaction4, SIGNAL(triggered(bool)), this, SLOT(assistantAction4(bool)));
-
-    KAction* assistantactionhide = actions->addAction("assistant_action_hide");
-    assistantactionhide->setText( i18n("&Hide Assistant") );
-    assistantactionhide->setShortcut( Qt::ALT | Qt::Key_0 );
-    connect(assistantactionhide, SIGNAL(triggered(bool)), this, SLOT(assistantHide()));
+    setupActions();
 }
 
 UiController::~UiController()
@@ -214,6 +222,22 @@ UiController::~UiController()
     delete d;
 }
 
+void UiController::setupActions()
+{
+#if (QT_VERSION >= QT_VERSION_CHECK(4, 8, 0))
+    if (d->defaultMainWindow->menuBar()->isNativeMenuBar()) {
+        KActionCollection* ac = d->defaultMainWindow->actionCollection();
+
+        // When running on an appmenu-enabled system, one cannot add widgets in the
+        // menubar corner. In this case, move area switcher to main toolbar.
+        KAction* switcherAction = new KAction(this);
+        switcherAction->setText(i18n("Area Switcher")); // For the toolbar edit dialog
+        switcherAction->setDefaultWidget(d->defaultMainWindow->areaSwitcher());
+        ac->addAction("area_switcher", switcherAction);
+    }
+#endif
+}
+
 void UiController::mainWindowDeleted(MainWindow* mw)
 {
     if (d->defaultMainWindow == mw)
@@ -262,7 +286,7 @@ void UiController::switchToArea(const QString &areaName, SwitchMode switchMode)
 
 QWidget* UiController::findToolView(const QString& name, IToolViewFactory *factory, FindFlags flags)
 {
-    if(!d->areasRestored)
+    if(!d->areasRestored || !activeArea())
         return 0;
 
     QList< Sublime::View* > views = activeArea()->toolViews();
@@ -295,6 +319,20 @@ QWidget* UiController::findToolView(const QString& name, IToolViewFactory *facto
     return ret;
 }
 
+void UiController::raiseToolView(QWidget* toolViewWidget)
+{
+    if(!d->areasRestored)
+        return;
+
+    QList< Sublime::View* > views = activeArea()->toolViews();
+    foreach(Sublime::View* view, views) {
+        if(view->widget() == toolViewWidget) {
+            view->requestRaise();
+            return;
+        }
+    }
+}
+
 void UiController::addToolView(const QString & name, IToolViewFactory *factory)
 {
     kDebug() ;
@@ -365,53 +403,66 @@ void UiController::cleanup()
     saveAllAreas(KGlobal::config());
 }
 
-void UiController::addNewToolView(MainWindow *mw)
+void UiController::selectNewToolViewToAdd(MainWindow *mw)
 {
     if (!mw || !mw->area())
         return;
     KDialog *dia = new KDialog(mw);
     dia->setCaption(i18n("Select Tool View to Add"));
     dia->setButtons(KDialog::Ok | KDialog::Cancel);
-    QListWidget *list = new QListWidget(dia);
+    NewToolViewListWidget *list = new NewToolViewListWidget(mw, dia);
 
+    list->setSelectionMode(QAbstractItemView::ExtendedSelection);
     list->setSortingEnabled(true);
     for (QMap<IToolViewFactory*, Sublime::ToolDocument*>::const_iterator it = d->factoryDocuments.constBegin();
         it != d->factoryDocuments.constEnd(); ++it)
     {
         ViewSelectorItem *item = new ViewSelectorItem(it.value()->title(), list);
         item->factory = it.key();
+        if (!item->factory->allowMultiple() && toolViewPresent(it.value(), mw->area())) {
+            // Disable item if the toolview is already present.
+            item->setFlags(item->flags() & ~Qt::ItemIsEnabled);
+        }
         list->addItem(item);
     }
 
     list->setFocus();
+    connect(list, SIGNAL(addNewToolView(MainWindow*,QListWidgetItem*)), this, SLOT(addNewToolView(MainWindow*,QListWidgetItem*)));
     dia->setMainWidget(list);
-    if (dia->exec() == QDialog::Accepted && list->currentItem())
+    if (dia->exec() == QDialog::Accepted)
     {
-        ViewSelectorItem *current = static_cast<ViewSelectorItem*>(list->currentItem());
-        Sublime::ToolDocument *doc = d->factoryDocuments[current->factory];
-        Sublime::View *view = doc->createView();
-        mw->area()->addToolView(view,
-            Sublime::dockAreaToPosition(current->factory->defaultPosition()));
-        current->factory->viewCreated(view);
+        foreach (QListWidgetItem* item, list->selectedItems())
+        {
+            addNewToolView(mw, item);
+        }
     }
     delete dia;
 }
 
+void UiController::addNewToolView(MainWindow *mw, QListWidgetItem* item)
+{
+    ViewSelectorItem *current = static_cast<ViewSelectorItem*>(item);
+    Sublime::ToolDocument *doc = d->factoryDocuments[current->factory];
+    Sublime::View *view = doc->createView();
+    mw->area()->addToolView(view,
+        Sublime::dockAreaToPosition(current->factory->defaultPosition()));
+    current->factory->viewCreated(view);
+}
+
 void UiController::showSettingsDialog()
 {
     QStringList blacklist = d->core->pluginControllerInternal()->projectPlugins();
-    kDebug() << "blacklist" << blacklist;
-    if(!d->cfgDlg)
-    {
-        d->cfgDlg = new KSettings::Dialog( QStringList() << "kdevplatform",
-                                           activeMainWindow() );
-        d->cfgDlg->setComponentBlacklist( blacklist );
+    foreach(const KPluginInfo& info, d->core->pluginControllerInternal()->allPluginInfos()) {
+        if (!blacklist.contains(info.pluginName()) && !info.isPluginEnabled()) {
+            blacklist << info.pluginName();
+        }
     }
-// The following doesn't work for some reason if the parent != activeMainWin,
-// the show() call doesn't show the dialog
-//     if( d->cfgDlg->dialog()->parentWidget() != activeMainWindow() )
-//         d->cfgDlg->dialog()->setParent( activeMainWindow() );
-    d->cfgDlg->exec();
+    kDebug() << "blacklist" << blacklist;
+    KSettings::Dialog cfgDlg( QStringList() << "kdevplatform",
+                                        activeMainWindow() );
+    cfgDlg.setMinimumSize( 550, 300 );
+    cfgDlg.setComponentBlacklist( blacklist );
+    cfgDlg.exec();
 }
 
 Sublime::Controller* UiController::controller()
@@ -427,14 +478,20 @@ KParts::MainWindow *UiController::activeMainWindow()
 void UiController::saveArea(Sublime::Area * area, KConfigGroup & group)
 {
     area->save(group);
-    Core::self()->workingSetControllerInternal()->getWorkingSet(area->workingSet())->saveFromArea(area, area->rootIndex());
+    if (!area->workingSet().isEmpty()) {
+        WorkingSet* set = Core::self()->workingSetControllerInternal()->getWorkingSet(area->workingSet());
+        set->saveFromArea(area, area->rootIndex());
+    }
 }
 
 void UiController::loadArea(Sublime::Area * area, const KConfigGroup & group)
 {
     area->load(group);
-    WorkingSet* set = Core::self()->workingSetControllerInternal()->getWorkingSet(area->workingSet());
-    Q_ASSERT(set->isConnected(area));
+    if (!area->workingSet().isEmpty()) {
+        WorkingSet* set = Core::self()->workingSetControllerInternal()->getWorkingSet(area->workingSet());
+        Q_ASSERT(set->isConnected(area));
+        Q_UNUSED(set);
+    }
 }
 
 void UiController::saveAllAreas(KSharedConfig::Ptr config)
@@ -548,14 +605,18 @@ void UiController::loadAllAreas(KSharedConfig::Ptr config)
     d->areasRestored = true;
 }
 
-void UiController::addToolViewToDockArea(const QString& name,
-                                         IToolViewFactory* factory,
-                                         Qt::DockWidgetArea area)
+void UiController::addToolViewToDockArea(IToolViewFactory* factory, Qt::DockWidgetArea area)
 {
-    ///TODO: we should probably add a bool forcePosition member to
-    /// Area::addToolView(), to force adding at the given position.
-    Sublime::View* view = addToolViewToArea(factory, d->factoryDocuments[factory], activeArea());
-    activeArea()->moveToolView(view, Sublime::dockAreaToPosition(area));
+    addToolViewToArea(factory, d->factoryDocuments[factory], activeArea(), Sublime::dockAreaToPosition(area));
+}
+
+bool UiController::toolViewPresent(Sublime::ToolDocument* doc, Sublime::Area* area)
+{
+    foreach (Sublime::View *view, doc->views()) {
+        if( area->toolViews().contains( view ) )
+            return true;
+    }
+    return false;
 }
 
 void UiController::addToolViewIfWanted(IToolViewFactory* factory,
@@ -570,12 +631,12 @@ void UiController::addToolViewIfWanted(IToolViewFactory* factory,
 
 Sublime::View* UiController::addToolViewToArea(IToolViewFactory* factory,
                                      Sublime::ToolDocument* doc,
-                                     Sublime::Area* area)
+                                     Sublime::Area* area, Sublime::Position p)
 {
     Sublime::View* view = doc->createView();
     area->addToolView(
         view,
-        Sublime::dockAreaToPosition(factory->defaultPosition()));
+        p == Sublime::AllPositions ? Sublime::dockAreaToPosition(factory->defaultPosition()) : p);
 
     connect(view, SIGNAL(raise(Sublime::View*)),
             SLOT(raiseToolView(Sublime::View*)));
@@ -644,25 +705,6 @@ const QMap< IToolViewFactory*, Sublime::ToolDocument* >& UiController::factoryDo
     return d->factoryDocuments;
 }
 
-void UiController::assistantAction1() {
-    if(d->currentShownAssistant)
-        d->currentShownAssistant->executeAction1();
-}
-
-void UiController::assistantAction2(bool) {
-    if(d->currentShownAssistant)
-        d->currentShownAssistant->executeAction2();
-}
-
-void UiController::assistantAction3(bool) {
-    if(d->currentShownAssistant)
-        d->currentShownAssistant->executeAction3();
-}
-void UiController::assistantAction4(bool) {
-    if(d->currentShownAssistant)
-        d->currentShownAssistant->executeAction4();
-}
-
 void UiController::assistantHide() {
     if(d->currentShownAssistant)
         hideAssistant(d->currentShownAssistant->assistant());
@@ -673,11 +715,8 @@ void UiController::assistantActionsChanged() {
         popUpAssistant(d->currentShownAssistant->assistant());
 }
 
-
 }
 
 
-
-
-
 #include "uicontroller.moc"
+#include "moc_uicontroller.cpp"
