@@ -27,6 +27,8 @@
 #include <ksharedconfig.h>
 #include <interfaces/iassistant.h>
 
+class QListWidgetItem;
+
 namespace Sublime {
     class AreaIndex;
     class ToolDocument;
@@ -67,7 +69,7 @@ public:
     virtual QWidget* findToolView(const QString& name, IToolViewFactory *factory, FindFlags flags);
     virtual void raiseToolView(QWidget* toolViewWidget);
 
-    void addNewToolView(MainWindow *mw);
+    void selectNewToolViewToAdd(MainWindow *mw);
 
     void initialize();
     void cleanup();
@@ -97,7 +99,9 @@ public:
 
     /// Adds a tool view in the active area to the dock area @p area.
     /// @see activeArea()
-    void addToolViewToDockArea(const QString& name, IToolViewFactory* factory, Qt::DockWidgetArea area);
+    void addToolViewToDockArea(KDevelop::IToolViewFactory* factory, Qt::DockWidgetArea area);
+
+    bool toolViewPresent(Sublime::ToolDocument* doc, Sublime::Area* area);
 
 private:
     void addToolViewIfWanted(IToolViewFactory* factory,
@@ -105,7 +109,8 @@ private:
                            Sublime::Area* area);
     Sublime::View* addToolViewToArea(IToolViewFactory* factory,
                            Sublime::ToolDocument* doc,
-                           Sublime::Area* area);
+                           Sublime::Area* area, Sublime::Position p=Sublime::AllPositions);
+    void setupActions();
 
 public Q_SLOTS:
     void raiseToolView(Sublime::View * view);
@@ -113,6 +118,7 @@ public Q_SLOTS:
 private Q_SLOTS:
     void assistantHide();
     void assistantActionsChanged();
+    void addNewToolView(MainWindow* mw, QListWidgetItem* item);
 
 private:
     class UiControllerPrivate* const d;
