@@ -32,6 +32,7 @@ namespace Ui { class BranchDialogBase; }
 namespace KDevelop
 {
     class DistributedVersionControlPlugin;
+class IBranchingVersionControl;
 }
 
 class BranchManager : public KDialog
@@ -61,25 +62,4 @@ private:
     
 };
 
-class BranchesListModel : public QStandardItemModel
-{
-    Q_OBJECT
-    public:
-        BranchesListModel(KDevelop::DistributedVersionControlPlugin* dvcsplugin, const QString& repo, QObject* parent = 0);
-        
-        void createBranch(const QString& baseBranch, const QString& newBranch);
-        void removeBranch(const QString& branch);
-        
-        KDevelop::DistributedVersionControlPlugin* dvcsPlugin() const { return dvcsplugin; }
-        QString repository() const { return repo; }
-        
-    public slots:
-        void resetCurrent();
-        
-    private:
-        KDevelop::DistributedVersionControlPlugin* dvcsplugin;
-        QString repo;
-};
-
-
 #endif
